@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 - 2000, 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -51,7 +51,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: klog.c,v 1.29.2.3 2002/07/11 14:00:38 lha Exp $");
+RCSID("$Id: klog.c,v 1.38 2003/06/04 11:46:31 hin Exp $");
 
 #include "appl_locl.h"
 #include "klog.h"
@@ -177,11 +177,12 @@ int
 get_afs_id(void)
 {
     int32_t returned_id;
+    const char **servers = (const char **)arg_servers.strings; /* ARGH */
 
     arlalib_get_token_id_servers (arg_principal,
 				  arg_cell,
 				  arg_servers.num_strings,
-				  arg_servers.strings,
+				  servers,
 				  &returned_id);
     return returned_id;
 }
@@ -421,7 +422,7 @@ struct agetargs args[] = {
       "principal to obtain token for",
       "user name", aarg_optional},
     { "password", 0, aarg_string, &arg_password,
-      "password to use (NOT RECOMENDED TO USE)",
+      "password to use (NOT RECOMMENDED TO USE)",
       "AFS password", aarg_optional},
     { "servers", 0, aarg_strings, &arg_servers,
       "list of servers to contact",
@@ -498,7 +499,7 @@ main(int argc, char **argv)
 
     char pwbuf[PASSWD_MAX];
 
-    setprogname (argv[0]);
+    set_progname (argv[0]);
 
     method = log_open (getprogname(), "/dev/stderr:notime");
     if (method == NULL)
@@ -651,9 +652,7 @@ main(int argc, char **argv)
 	if(arg_pipe) {
 	    if (fgets (pwbuf, sizeof(pwbuf), stdin) == NULL)
 		dietx (1, "EOF reading password");
-	    if (pwbuf[strlen(pwbuf) - 1] != '\n')
-		dietx (1, "too long password");
-	    pwbuf[strlen(pwbuf) - 1] = '\0';
+	    pwbuf[strcspn(pwbuf, "\n")] = '\0';
 	} else {
 	    snprintf(prompt, PW_PROMPT_MAX, 
 		     "%s@%s's Password:", arg_principal, arg_cell);
