@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: arlalib.h,v 1.37.2.3 2001/10/02 16:13:01 jimmy Exp $ */
+/* $Id: arlalib.h,v 1.50 2003/04/24 11:51:24 lha Exp $ */
 
 #ifndef ARLALIB_H
 #define ARLALIB_H 1
@@ -40,11 +40,13 @@
  * Credentinals
  */
 
-typedef enum { AUTHFLAGS_NOAUTH = 0,
-	      AUTHFLAGS_ANY = 1,
-	      AUTHFLAGS_LOCALAUTH = 2,
-	      AUTHFLAGS_TICKET = 4,
-	      AUTHFLAGS_TOKEN = 8 } arlalib_authflags_t;
+typedef enum { AUTHFLAGS_NOAUTH		= 0x0,
+	       AUTHFLAGS_ANY		= 0x1,
+	       AUTHFLAGS_LOCALAUTH	= 0x2,
+	       AUTHFLAGS_TICKET		= 0x4,
+	       AUTHFLAGS_TOKEN		= 0x8,
+	       AUTHFLAGS_DISALLOW_NOAUTH = 0x10
+} arlalib_authflags_t;
 
 arlalib_authflags_t 
 arlalib_getauthflag (int noauth,
@@ -67,16 +69,16 @@ arlalib_getconnbyname(const char *cell, const char *host,
 		      arlalib_authflags_t auth);
 
 int arlalib_destroyconn(struct rx_connection *conn);
-int arlalib_getservername(u_int32_t serverNumber, char **servername);
+int arlalib_getservername(uint32_t serverNumber, char **servername);
 struct rx_securityClass*
 arlalib_getsecurecontext(const char *cell, const char *host, 
-			 arlalib_authflags_t auth);
+			 arlalib_authflags_t auth, int *secidx);
 int arlalib_getsyncsite(const char *cell, const char *host, int32_t port, 
-			u_int32_t *synchost, arlalib_authflags_t auth);
+			uint32_t *synchost, arlalib_authflags_t auth);
 
 
 /*
- * Token managment
+ * Token management
  */
 
 struct ClearToken;
@@ -103,10 +105,8 @@ int fs_nop(void);
 
 /* arla extensions */
 
-const char *fslib_version(void);
-
-int fs_setcrypt (u_int32_t level);
-int fs_getcrypt (u_int32_t *level);
+int fs_setcrypt (uint32_t level);
+int fs_getcrypt (uint32_t *level);
 
 int fs_connect (int32_t type, int32_t *flags);
 
@@ -117,17 +117,21 @@ int fs_getmaxfprio(int16_t *maxprio);
 
 int fs_gcpags(void);
 
-int fs_calculate_cache(u_int32_t *calculated,
-		       u_int32_t *usedbytes);
+int fs_calculate_cache(uint32_t *calculated,
+		       uint32_t *usedbytes);
 
-int fs_getfilecachestats(u_int32_t *max_bytes, u_int32_t *used_bytes,
-			 u_int32_t *max_vnodes, u_int32_t *used_vnodes);
+int fs_getfilecachestats(int64_t *max_bytes,
+			 int64_t *used_bytes,
+			 int64_t *low_bytes,
+			 int64_t *max_vnodes,
+			 int64_t *used_vnodes,
+			 int64_t *low_vnodes);
 
-int fs_getaviatorstats(u_int32_t *max_workers,
-		       u_int32_t *used_workers);
+int fs_getaviatorstats(uint32_t *max_workers,
+		       uint32_t *used_workers);
 
 int fs_checkservers(char *cell, int32_t flags,
-		    u_int32_t *hosts, int numhosts);
+		    uint32_t *hosts, int numhosts);
 
 int fs_checkvolumes (void);
 
@@ -156,11 +160,11 @@ int
 fs_newcell (const char *cell, int nservers, char **servers);
 
 int
-fs_getcells (int32_t num, u_int32_t *server, int numservers,
+fs_getcells (int32_t num, uint32_t *server, int numservers,
 	     char *cell, size_t cell_sz);
 
 int
-fs_getcellstatus (char *cellname, u_int32_t *flags);
+fs_getcellstatus (char *cellname, uint32_t *flags);
 
 int
 fs_invalidate (const char *path);
@@ -175,11 +179,11 @@ int
 fs_incompat_renumber (int *ret);
 
 int
-fs_statistics_list(u_int32_t *host, u_int32_t *part, int *n);
+fs_statistics_list(uint32_t *host, uint32_t *part, int *n);
 
 int
-fs_statistics_entry(u_int32_t host, u_int32_t part, u_int32_t type,
-		    u_int32_t items_slot, u_int32_t *count,
+fs_statistics_entry(uint32_t host, uint32_t part, uint32_t type,
+		    uint32_t items_slot, uint32_t *count,
 		    int64_t *items_total, int64_t *total_time);
 
 int
@@ -191,8 +195,8 @@ arlalib_get_viceid_servers (const char *username, const char *cellname,
 			    int nservers, const char *servers[],
 			    int32_t *viceId);
 
-int xfs_debug(int inflags, int *outflags);
-int xfs_debug_print(int inflags);
+int nnpfs_debug(int inflags, int *outflags);
+int nnpfs_debug_print(int inflags, char *);
 int arla_debug(int inflags, int *outflags);
 
 /* db server context */
@@ -231,4 +235,13 @@ arlalib_get_token_id_servers (const char *username, const char *cellname,
 			      int nservers, const char *servers[],
 			      int32_t *token_id);
 
+void
+arlalib_host_to_name (uint32_t addr, char *str, size_t str_sz);
+
+int
+arlalib_name_to_host (const char *str, uint32_t *addr);
+
+int
+arlalib_version_cmd(int argc, char **argv);
+
 #endif /* ARLALIB_H */
