@@ -36,26 +36,30 @@
  * (KAA,KAT,KAM) that is available from ka-server.
  */
 
+/* #define KASERVER_SUPPORT 1 */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-RCSID("$Id: ka-procs.c,v 1.14 2003/10/03 17:30:33 map Exp $");
-
-#if 0
+RCSID("$Id: ka-procs.c,v 1.14.2.4 2004/06/27 22:11:58 lha Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #include <assert.h>
 
+#include <ka-procs.h>
+
+#ifdef KASERVER_SUPPORT
+
 #ifdef HAVE_OPENSSL
 #include <openssl/des.h>
 #else
 #include <des.h>
 #endif
-#include <krb.h>
 
 #include <ko.h>
 #include <ports.h>
@@ -72,10 +76,31 @@ RCSID("$Id: ka-procs.c,v 1.14 2003/10/03 17:30:33 map Exp $");
 #include <ka.h>
 #include <ka.cs.h>
 
-#include <ka-procs.h>
-
 #include <kafs.h>
 
+typedef struct {
+    char dat[MAXKTCTICKETLEN];
+    size_t length;
+} ka_ticket_t;
+
+struct ka_Answer {
+    uint32_t challange;
+    unsigned char sessionkey[8];
+    uint32_t start_time;
+    uint32_t end_time;
+    int kvno;
+    char user[MAXKANAMELEN];
+    char instance[MAXKANAMELEN];
+    char realm[MAXKANAMELEN];
+    char server_user[MAXKANAMELEN];
+    char server_instance[MAXKANAMELEN];
+    ka_ticket_t ticket;
+    char label[KA_LABELSIZE];
+};
+
+typedef struct ka_Answer ka_auth_data_t;
+typedef struct ka_Answer ka_ticket_data_t;
+
 /*
  * The ka preauth data-blob. Used in KAA_Authenticate. Encrypted with
  * des_pcbc_encrypt() and the user's key.
@@ -97,6 +122,7 @@ struct ka_times {
     int32_t end_time;
 };
 
+
 /*
  *
  */
@@ -165,7 +191,7 @@ decode_answer (char *label, unsigned char *buf, size_t sz,
     decode_u_int32 (&kvno_int32, &buf, &sz);
     answer->kvno = (int)kvno_int32;
     decode_u_int32 (&ticket_sz, &buf, &sz);
-    if (ticket_sz > MAX_KTXT_LEN)
+    if (ticket_sz > MAXKTCTICKETLEN)
 	goto fail;
     
 #define destrz(f) decode_stringz(answer->f,sizeof(answer->f), &buf, &sz)
@@ -181,7 +207,6 @@ decode_answer (char *label, unsigned char *buf, size_t sz,
 
     answer->ticket.length = ticket_sz;
     memcpy (&answer->ticket.dat, buf, ticket_sz);
-    answer->ticket.mbz = 0;
     sz -= ticket_sz;
     buf += ticket_sz;
     
@@ -202,6 +227,75 @@ decode_answer (char *label, unsigned char *buf, size_t sz,
     return 1;
 }
 
+#define lowcase(c) (('A' <= (c) && (c) <= 'Z') ? ((c) - 'A' + 'a') : (c))
+
+/*
+ * The string to key function used by Transarc AFS. Stolen from kth-krb.
+ */
+static void
+afs_string_to_key(const char *pass, const char *cell, des_cblock *key)
+{
+  if (strlen(pass) <= 8)	/* Short passwords. */
+    {
+      char buf[8 + 1], *s;
+      int i;
+
+      /*
+       * XOR cell and password and pad (or fill) with 'X' to length 8,
+       * then use crypt(3) to create DES key.
+       */
+      for (i = 0; i < 8; i++)
+	{
+	  buf[i] = *pass ^ lowcase(*cell);
+	  if (buf[i] == 0)
+	    buf[i] = 'X';
+	  if (*pass != 0)
+	    pass++;
+	  if (*cell != 0)
+	    cell++;
+	}
+      buf[8] = 0;
+
+      s = crypt(buf, "p1");	/* Result from crypt is 7bit chars. */
+      s = s + 2;		/* Skip 2 chars of salt. */
+      for (i = 0; i < 8; i++)
+	((char *) key)[i] = s[i] << 1; /* High bit is always zero */
+      des_fixup_key_parity(key); /*       Low  bit is parity */
+    }
+  else				/* Long passwords */
+    {
+      int plen, clen;
+      char *buf, *t;
+      des_key_schedule sched;
+      des_cblock ivec;
+
+      /*
+       * Concatenate password with cell name,
+       * then checksum twice to create DES key.
+       */
+      plen = strlen(pass);
+      clen = strlen(cell);
+      buf = malloc(plen + clen + 1);
+      memcpy(buf, pass, plen);
+      for (t = buf + plen; *cell != 0; t++, cell++)
+	*t = lowcase(*cell);
+
+      memcpy(&ivec, "kerberos", 8);
+      memcpy(key, "kdsbdsns", 8);
+      des_key_sched(key, sched);
+      /* Beware, ivec is passed twice */
+      des_cbc_cksum((des_cblock*)buf, &ivec, plen + clen, sched, &ivec);
+
+      memcpy(key, &ivec, 8);
+      des_fixup_key_parity(key);
+      des_key_sched(key, sched);
+      /* Beware, ivec is passed twice */
+      des_cbc_cksum((des_cblock*)buf, key, plen + clen, sched, &ivec);
+      free(buf);
+      des_fixup_key_parity(key);
+    }
+}
+
 /*
  * Get the the password des-`key' for `realm' (and its make sure that
  * the cell is lowercase) using `password'. If `password' is NULL, the
@@ -244,99 +338,19 @@ fixup_realm (struct ka_Answer *answer, const char *realm)
     }
 }
 
-/*
- * Authenticate `user'.`instance'@`cell' with `password' and get
- * `lifetime' second long tickets. If `password' is NULL, user is
- * queried for password. `flags' is used to specify behavior of the
- * function.
- */
-
-int
-ka_authenticate (const char *user, const char *instance, const char *cell,
-		 const char *password, uint32_t lifetime, 
-		 ka_auth_flags_t flags)
+static void
+build_cleartoken(ka_ticket_data_t *tdata, struct ClearToken *ct)
 {
-    des_cblock key;
-    ka_auth_data_t adata;
-    ka_ticket_data_t tdata;
-    struct ka_cell_query cinfo;
-    char filename[MAXPATHLEN];
-    int ret;
-    int save_ticket, do_afslog = 0;
+    ct->AuthHandle = tdata->kvno;
+    memcpy(ct->HandShakeKey,
+	   tdata->sessionkey, sizeof(tdata->sessionkey));
+    ct->ViceId = 0;
+    ct->BeginTimestamp = tdata->start_time;
+    ct->EndTimestamp = tdata->end_time;
 
-    filename[0] = '\0';
-    if (cell == NULL)
-	cell = cell_getthiscell();
-    if (instance == NULL)
-	instance = "";
-
-    memset (&cinfo, 0, sizeof(cinfo));
-
-    save_ticket = (flags & KA_AUTH_TICKET) == KA_AUTH_TICKET;
-    do_afslog = (flags & KA_AUTH_TOKEN) == KA_AUTH_TOKEN;
-
-    ret = get_password (cell, password, &key);
-    if (ret)
-	goto out;
-
-    ret = ka_auth (user, instance, cell, &key, &adata, lifetime,
-		   &cinfo);
-    if (ret)
-	goto out;
-    
-    if (save_ticket) {
-	strlcpy (filename, tkt_string(), sizeof(filename));
-    } else if (do_afslog) {
-	snprintf (filename, sizeof(filename), "%sXXXXXXXX", tkt_string());
-	ret = mkstemp (filename);
-	if (ret) {
-	    ret = errno;
-	    goto out;
-	}
-    }
-    
-    if (save_ticket || do_afslog) {
-	ret = ka_auth_create (filename, &adata);
-	if (ret)
-	    goto out;
-    }
-	
-    if (do_afslog) {
-	ret = ka_getticket ("afs", "", cell, &adata, &tdata, &cinfo);
-	if (ret)
-	    goto out;
-
-	ret = ka_write_ticket (filename, &tdata);
-	if (ret)
-	    goto out;
-
-	if (k_hasafs()) {
-#ifdef HAVE_KRB_AFSLOG_UID
-	    ret = krb_afslog (cell, cell);
-#else
-	    ret = k_afsklog_uid (cell, cell);
-#endif
-	    if (ret) {
-		warnx ("failed inserting tokens for cell %s", cell);
-	    }
-	    ret = 0;
-	}
-    }
-
-    if (!save_ticket)
-	dest_tkt();
-	
- out:
-    memset (&key, 0, sizeof (key));
-    memset (&adata, 0, sizeof(adata));
-    memset (&tdata, 0, sizeof(tdata));
-    memset (&cinfo, 0, sizeof(cinfo));
-    if (ret) {
-	if (filename[0] != '\0')
-	    unlink(filename);
-    }
-
-    return ret;
+    /* traditional: diff must be even */
+    if ((ct->EndTimestamp - ct->BeginTimestamp) & 1)
+	ct->EndTimestamp--;
 }
 
 #ifndef DES_ENCRYPT
@@ -354,13 +368,13 @@ ka_authenticate (const char *user, const char *instance, const char *cell,
 /*
  * Authenticate `user'.`instance'@`cell' with `key' to get a tgt for
  * with a lifetime `lifetime' seconds. Return ticket in
- * `adata'. `cinfo' is used to specify how to query cell.
+ * `adata'.
  */
 
 int
 ka_auth (const char *user, const char *instance, const char *cell,
 	 des_cblock *key, ka_auth_data_t *adata,
-	 uint32_t lifetime, struct ka_cell_query *cinfo)
+	 uint32_t lifetime)
 {
     struct db_server_context conn_context;
     struct rx_connection *conn;
@@ -402,8 +416,8 @@ ka_auth (const char *user, const char *instance, const char *cell,
     req_challange = tv.tv_sec;
 
     des_set_key (key, schedule);
-    des_pcbc_encrypt ((unsigned char *)&f, 
-		      (unsigned char *)buf,
+    des_pcbc_encrypt ((des_cblock *)&f, 
+		      (des_cblock *)buf,
 		      sizeof(f), 
 		      schedule,
 		      key, 
@@ -445,8 +459,8 @@ ka_auth (const char *user, const char *instance, const char *cell,
      */
 
     des_set_key (key, schedule);
-    des_pcbc_encrypt ((unsigned char *)answer.Seq.val,
-		      (unsigned char *)answer.Seq.val,
+    des_pcbc_encrypt ((des_cblock *)answer.Seq.val,
+		      (des_cblock *)answer.Seq.val,
 		      answer.Seq.len,
 		      schedule,
 		      key,
@@ -484,14 +498,12 @@ ka_auth (const char *user, const char *instance, const char *cell,
 
 /*
  * Get a ticket for `suser'.`sinstance'@`srealm' using `adata' to
- * authenticate. The returning ticket is retrurned in `tdata'. `cinfo'
- * is used for special cell handling.
+ * authenticate. The returning ticket is retrurned in `tdata'.
  */
 
 int
 ka_getticket (const char *suser, const char *sinstance, const char *srealm,
-	      ka_auth_data_t *adata, ka_ticket_data_t *tdata,
-	      struct ka_cell_query *cinfo)
+	      ka_auth_data_t *adata, ka_ticket_data_t *tdata)
 {
     struct db_server_context conn_context;
     struct rx_connection *conn;
@@ -556,8 +568,8 @@ ka_getticket (const char *suser, const char *sinstance, const char *srealm,
 
     /* decrypt it */
     des_set_key (&adata->sessionkey, schedule);
-    des_pcbc_encrypt ((unsigned char *)answer.Seq.val,
-		      (unsigned char *)answer.Seq.val,
+    des_pcbc_encrypt ((des_cblock *)answer.Seq.val,
+		      (des_cblock *)answer.Seq.val,
 		      answer.Seq.len,
 		      schedule,
 		      &adata->sessionkey,
@@ -571,62 +583,59 @@ ka_getticket (const char *suser, const char *sinstance, const char *srealm,
     return 0;
 }
 
-/*
- * Create a new credential cache `filename' and add the TGT `data'.
- */
-
-int
-ka_auth_create (char *filename, ka_auth_data_t *data)
-{
-    int ret;
-    
-    unlink (filename);
-
-    ret = tf_create (filename);
-    if (ret)
-	return ret;
-    
-    if (tf_put_pname(data->user) != KSUCCESS ||
-	tf_put_pinst(data->instance) != KSUCCESS) {
-	tf_close();
-	return INTK_ERR;
-    }
-    
-    ret = tf_save_cred (data->server_user, 
-			data->server_instance, 
-			data->realm, 
-			data->sessionkey,
-			data->end_time - data->start_time, 
-			data->kvno, 
-			&data->ticket, 
-			data->start_time);
-    tf_close();
-    return ret;
-}
-
-/*
- * Append new ticket `data' to the existing cred cache `filename'
- */
-
-int
-ka_write_ticket (char *filename, ka_ticket_data_t *data)
-{
-    int ret;
-
-    ret = tf_init(filename, W_TKT_FIL);
-    if (ret)
-	return ret;
-    
-    ret = tf_save_cred (data->server_user, 
-			data->server_instance, 
-			data->realm, 
-			data->sessionkey,
-			data->end_time - data->start_time, 
-			data->kvno, 
-			&data->ticket, 
-			data->start_time);
-    tf_close();
-    return ret;
-}
-
 #endif
+
+/*
+ * Authenticate `user'.`instance'@`cell' with `password' and get
+ * `lifetime' second long tickets. If `password' is NULL, user is
+ * queried for password. `flags' is used to specify behavior of the
+ * function.
+ */
+
+int
+ka_authenticate (const char *user, const char *instance, const char *cell,
+		 const char *password, uint32_t lifetime)
+{
+#ifdef KASERVER_SUPPORT
+    des_cblock key;
+    ka_auth_data_t adata;
+    ka_ticket_data_t tdata;
+    struct ClearToken ct;
+    int ret;
+
+    if (cell == NULL)
+	cell = cell_getthiscell();
+    if (instance == NULL)
+	instance = "";
+
+    ret = get_password (cell, password, &key);
+    if (ret)
+	goto out;
+
+    ret = ka_auth (user, instance, cell, &key, &adata, lifetime);
+    if (ret)
+	goto out;
+
+    ret = ka_getticket ("afs", "", cell, &adata, &tdata);
+    if (ret)
+	goto out;
+
+    if (k_hasafs()) {
+	build_cleartoken(&tdata, &ct);
+	ret = kafs_settoken_rxkad(cell, &ct, &tdata.ticket.dat,
+				  tdata.ticket.length);
+	if (ret)
+	    warnx ("failed inserting tokens for cell %s", cell);
+    }
+
+ out:
+    memset (&key, 0, sizeof (key));
+    memset (&adata, 0, sizeof(adata));
+    memset (&tdata, 0, sizeof(tdata));
+
+    return ret;
+#else
+    return EINVAL;
+#endif
+}
+
