@@ -33,7 +33,7 @@
 
 #include "appl_locl.h"
 
-RCSID("$Id: tokens.c,v 1.1.2.4 2002/08/29 19:57:43 ahltorp Exp $");
+RCSID("$Id: tokens.c,v 1.9 2003/02/01 17:01:11 lha Exp $");
 
 /*
  * Iterate over all tokens and apply `func' on all of them, passing
@@ -57,7 +57,7 @@ arlalib_token_iter (const char *cell, arlalib_token_iter_func func, void *arg)
 	return ENOSYS;
 
     for (i = 0;; i++) {
-	u_int32_t sz;
+	uint32_t sz;
 	unsigned char *t = data;
 
 	parms.in = (void *)&i;
@@ -68,7 +68,7 @@ arlalib_token_iter (const char *cell, arlalib_token_iter_func func, void *arg)
 	if(k_pioctl(NULL, VIOCGETTOK, &parms, 0) != 0) {
 	    if(errno == EDOM)
 		break;
-	    continue;
+	    return EINVAL;
 	}
 
 	/* secret token, v4 or v5 ticket */
@@ -92,25 +92,18 @@ arlalib_token_iter (const char *cell, arlalib_token_iter_func func, void *arg)
 	memcpy (&ct, t, sz);
 	t += sz;
 
-	/* cell */
-	memcpy (&sz, t, sizeof(sz)); /* primary cell */
-	t += sizeof(sz);
-	if (parms.out_size < t - data + sz)
-	    continue;
+	t += 4; /* skip primary cell flag */
+	
+	/* cell name */
+	data[sizeof(data) - 1] = '\0'; /* it should be zero terminated, 
+					  but make sure it is */
 	strlcpy(token_cell, t, sizeof(token_cell));
 	
-	if (cell) {
-	    if (strcasecmp(token_cell, cell) == 0) {
-		ret = (*func) (secret, secret_sz, &ct, cell, arg);
-		memset (data, 0, sizeof(data));
-		return ret;
-	    }
-	} else {
+	if (cell == NULL || strcasecmp(token_cell, cell) == 0) {
 	    ret = (*func) (secret, secret_sz, &ct, token_cell, arg);
-	    if (ret) {
-		memset (data, 0, sizeof(data));
+	    memset (data, 0, sizeof(data));
+	    if (cell != NULL)
 		return ret;
-	    }
 	}
     }
     memset (data, 0, sizeof(data));
