@@ -87,7 +87,7 @@ checkdaemonpid(AuthorizationRef authorization, char *pid)
 }
 
 static int
-xfs_umount(AuthorizationRef authorization)
+nnpfs_umount(AuthorizationRef authorization)
 {
     char *argv[2];
     OSStatus status;
@@ -96,7 +96,7 @@ xfs_umount(AuthorizationRef authorization)
     argv[0] = "/afs";
     argv[1] = NULL;
 
-    status = AuthorizationExecuteWithPrivileges(authorization, "/usr/arla/sbin/umount_xfs", 0, argv, &output);
+    status = AuthorizationExecuteWithPrivileges(authorization, "/usr/arla/sbin/umount_nnpfs", 0, argv, &output);
     if (status == noErr) {
         drainfile(output);
         fclose(output);
@@ -108,17 +108,17 @@ xfs_umount(AuthorizationRef authorization)
 }
 
 static int
-xfs_mount(AuthorizationRef authorization)
+nnpfs_mount(AuthorizationRef authorization)
 {
     char *argv[3];
     OSStatus status;
     FILE *output;
 
-    argv[0] = "/dev/xfs0";
+    argv[0] = "/dev/nnpfs0";
     argv[1] = "/afs";
     argv[2] = NULL;
 
-    status = AuthorizationExecuteWithPrivileges(authorization, "/usr/arla/sbin/mount_xfs", 0, argv, &output);
+    status = AuthorizationExecuteWithPrivileges(authorization, "/usr/arla/sbin/mount_nnpfs", 0, argv, &output);
     if (status == noErr) {
         drainfile(output);
         fclose(output);
@@ -185,8 +185,7 @@ startarlad(AuthorizationRef authorization)
     FILE *output;
 
     argv[0] = "-D";
-    argv[1] = "--fake-stat";
-    argv[2] = NULL;
+    argv[1] = NULL;
 
     status = AuthorizationExecuteWithPrivileges(authorization, "/usr/arla/libexec/arlad", 0, argv, &output);
     if (status == noErr) {
@@ -206,7 +205,7 @@ kmodunload(AuthorizationRef authorization)
     OSStatus status;
     FILE *output;
 
-    argv[0] = "/usr/arla/bin/xfs.kext";
+    argv[0] = "/usr/arla/bin/nnpfs.kext";
     argv[1] = NULL;
 
     status = AuthorizationExecuteWithPrivileges(authorization, "/sbin/kextunload", 0, argv, &output);
@@ -227,7 +226,7 @@ kmodload(AuthorizationRef authorization)
     OSStatus status;
     FILE *output;
 
-    argv[0] = "/usr/arla/bin/xfs.kext";
+    argv[0] = "/usr/arla/bin/nnpfs.kext";
     argv[1] = NULL;
 
     status = AuthorizationExecuteWithPrivileges(authorization, "/sbin/kextload", 0, argv, &output);
@@ -689,12 +688,12 @@ row:(int)row {
 	t.tv_usec = 250000;
 	select(0, NULL, NULL, NULL, &t);
     }
-    xfs_mount(authorization);
+    nnpfs_mount(authorization);
     disktool_refresh(authorization);
 }
 
 + (void) stopDaemon: (AuthorizationRef) authorization {
-    xfs_umount(authorization);
+    nnpfs_umount(authorization);
     disktool_refresh(authorization);
     stoparlad(authorization);
     kmodunload(authorization);
