@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2003 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,8 +14,6 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- *      Hskolan and its contributors.
- * 
  * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
@@ -36,7 +34,7 @@
 #include <sl.h>
 #include "appl_locl.h"
 
-RCSID("$Id: pts.c,v 1.50.2.3 2002/07/11 14:00:41 lha Exp $");
+RCSID("$Id: pts.c,v 1.57 2003/10/04 02:36:32 lha Exp $");
 
 static int help_cmd (int argc, char **argv);
 
@@ -241,6 +239,7 @@ pr_name2id(struct rx_connection *conn, const char *name, int32_t *id)
     char rname[PR_MAXNAMELEN];
 
     assert(id);
+    *id = 0;
 
     if (prdebug)
 	printf("pr_name2id(%s, x)", name);
@@ -252,14 +251,17 @@ pr_name2id(struct rx_connection *conn, const char *name, int32_t *id)
 
     error = PR_NameToID(conn, &nlist, &ilist);
 
-    if (error || ilist.len != 1)
-	*id = 0;
-    else
-	*id = *ilist.val;
-	
     if (prdebug)
 	printf(" id = %d, error= %d\n", *id, error);
 
+    if (error)
+	return error;
+	
+    if (ilist.len == 1)
+	*id = *ilist.val;
+    else
+	error = PRBADARG;
+
     free(ilist.val);
     
     return error;
@@ -1277,6 +1279,8 @@ examine_cmd (int argc, char **argv)
 	error = examine1 (connptdb, &users);
     }
     free_db_server_context(&conn_context);
+    if (error)
+	warnx("PR_Examine failed: %s", koerr_gettext (error));
     return 0;
 }
 
@@ -1505,6 +1509,7 @@ static SL_cmd cmds[] = {
     {"setfields",   setfields_cmd,  "not yet implemented"},
     {"setmax",      empty_cmd,      "not yet implemented"},
     {"syncdb",      syncdb_cmd,     "sync ptsdb with /etc/passwd"},
+    {"version",	    arlalib_version_cmd, "print version"},
     {NULL}
 };
 
