@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,7 @@
  * Our cache of volume information.
  */
 
-/* $Id: fprio.h,v 1.4 1999/04/14 15:27:36 map Exp $ */
+/* $Id: fprio.h,v 1.6 2000/10/02 22:31:48 lha Exp $ */
 
 #ifndef _FPRIO_
 #define _FPRIO_
@@ -57,9 +52,9 @@ struct fpriorityentry {
 void fprio_init (char *file);
 int  fprio_clear(void);
 void fprio_remove(VenusFid fid);
-void fprio_set(VenusFid fid, unsigned prio);
+void fprio_set(VenusFid fid, Bool prio);
 int  fprio_readin(char *file);
-int  fprio_get(VenusFid fid);
+Bool fprio_get(VenusFid fid);
 void fprio_status (void);
 
 #endif /* _FPRIO_ */
