@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,24 +35,40 @@
  *
  */
 
-/* $Id: messages.h,v 1.10 1999/07/24 18:49:49 lha Exp $ */
+/* $Id: messages.h,v 1.24 2002/09/07 10:43:22 lha Exp $ */
 
 #ifndef _MESSAGES_H_
 #define _MESSAGES_H_
 
-void xfs_message_init (void);
-int xfs_message_receive (int fd, struct xfs_message_header *h, u_int size);
-void break_callback (VenusFid fid);
-void install_attr (FCacheEntry *e);
+void nnpfs_message_init (void);
+int nnpfs_message_receive (int fd, struct nnpfs_message_header *h, u_int size);
+void break_callback (FCacheEntry *e);
+void install_attr (FCacheEntry *e, int flags);
 
-long afsfid2inode(VenusFid *fid);
+long afsfid2inode(const VenusFid *fid);
 
 int
-xfs_attr2afsstorestatus(struct xfs_attr *xa,
+nnpfs_attr2afsstorestatus(struct nnpfs_attr *xa,
 			AFSStoreStatus *storestatus);
 
 void
-update_kernelfid(VenusFid oldfid, VenusFid newfid);
+update_fid(VenusFid oldfid, FCacheEntry *old_entry,
+	   VenusFid newfid, FCacheEntry *new_entry);
 
+enum { FCACHE2NNPFSNODE_LENGTH = 1 } ;	/* allow update of filedata */
+
+#define FCACHE2NNPFSNODE_NO_LENGTH	0
+#define FCACHE2NNPFSNODE_ALL		(FCACHE2NNPFSNODE_LENGTH)
+
+void
+fcacheentry2nnpfsnode (const VenusFid *fid,
+		     const VenusFid *statfid, 
+		     AFSFetchStatus *status,
+		     struct nnpfs_msg_node *node,
+                     AccessEntry *ae,
+		     int flags);
+
+int
+VenusFid_cmp (const VenusFid *fid1, const VenusFid *fid2);
 
 #endif /* _MESSAGES_H_ */
