@@ -21,7 +21,9 @@ or implied warranty.
 
 #include "config.h"
 
-RCSID("$Id: acl_files.c,v 1.4 1999/07/29 08:05:50 assar Exp $");
+RCSID("$Id: acl_files.c,v 1.5 1999/09/09 23:46:01 assar Exp $");
+
+#ifdef KERBEROS
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -605,3 +607,5 @@ acl_delete(char *acl, char *principal)
     }
     return(acl_commit(acl, new));
 }
+
+#endif /* KERBEROS */
