@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: agetarg.c,v 1.8.2.4 2002/06/11 14:10:39 ahltorp Exp $");
+RCSID("$Id: agetarg.c,v 1.13 2002/09/17 18:30:53 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -99,8 +99,8 @@ mandoc_template(struct agetargs *args,
     t = time(NULL);
     strftime(timestr, sizeof(timestr), "%b %d, %Y", localtime(&t));
     printf(".Dd %s\n", timestr);
-    p = strrchr(__progname, '/');
-    if(p) p++; else p = __progname;
+    p = strrchr(getprogname(), '/');
+    if(p) p++; else p = getprogname();
     strncpy(cmd, p, sizeof(cmd));
     cmd[sizeof(cmd)-1] = '\0';
     strupr(cmd);
@@ -174,7 +174,7 @@ aarg_printusage (struct agetargs *args,
     size_t max_len = 0;
 
     if (progname == NULL)
-	progname = __progname;
+	progname = getprogname();
 
     if(getenv("GETARGMANDOC")){
 	mandoc_template(args, extra_string, style);
@@ -426,6 +426,8 @@ arg_match_long(struct agetargs *args, int argc,
 	    current = partial;
 	else
 	    return AARG_ERR_NO_MATCH;
+
+	numarg = current - args;
     }
 
     if(*optarg == '\0' && !ISFLAG(current))
@@ -478,8 +480,8 @@ agetarg(struct agetargs *args,
 	    usedargs[swcount] = 1;
 	    i += j;
 	    swcount++;
-	} else if((argv[i][1] == '-' || 
-		   ((style & AARG_TRANSLONG) && argv[i][1] != 0))) {
+	} else if(argv[i][1] == '-' || 
+		  ((style & AARG_TRANSLONG) && argv[i][1] != 0)) {
 	    int k;
 
 	    if(argv[i][2] == 0 && !(style & AARG_TRANSLONG)){
@@ -557,8 +559,7 @@ agetarg(struct agetargs *args,
 		}
 	    }
 	out:;
-	} else
-	    break;
+	}
     }
     *optind = i;
 
