@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: koerror.c,v 1.20 2001/01/08 16:48:03 lha Exp $");
+RCSID("$Id: koerror.c,v 1.29 2003/11/04 22:25:53 tol Exp $");
 #endif
 
 #include <stdio.h>
@@ -41,21 +41,18 @@ RCSID("$Id: koerror.c,v 1.20 2001/01/08 16:48:03 lha Exp $");
 
 #include <roken.h>
 
+#include <rx/rx.h>
+#include <rx/rxgencon.h>
+#include <rxkad.h>
+#include <ko.h>
+#include <fs_errors.h>
+
 #include <vldb.h>
 #include <volumeserver.h>
 #include <pts.h>
 #include <bos.h>
 #include <ubik.h>
 #include <ka.h>
-#include <rx/rx.h>
-#include <rx/rxgencon.h>
-#ifdef KERBEROS
-#include <krb.h>
-#include <des.h>
-#include <rxkad.h>
-#endif
-#include <ko.h>
-#include <fs_errors.h>
 
 struct koerr {
     koerr_t code;
@@ -94,6 +91,10 @@ static struct koerr koerrmsg[] = {
     {VL_BADSERVERFLAG,  "VL - Invalid replication site server flag."},
     {VL_PERM,           "VL - No permission access."},
     {VL_NOMEM,          "VL - malloc(realloc) failed to alloc enough memory"},
+    {VL_BADVERSION,	"VL - Bad version"},
+    {VL_INDEXERANGE,	"VL - Index out of range"},
+    {VL_MULTIPADDR,	"VL - Multiple IP addresses"},
+    {VL_BADMASK,	"VL - Bad mask"},
 
     /* VOLSER errors */
 
@@ -150,7 +151,7 @@ static struct koerr koerrmsg[] = {
     {RXKADINCONSISTENCY,      "rxkad - Inconsistency."},
     {RXKADPACKETSHORT,        "rxkad - Packet too short."},
     {RXKADLEVELFAIL,          "rxkad - Security level failed."},
-    {RXKADTICKETLEN,          "rxkad - Invaild ticket length."},
+    {RXKADTICKETLEN,          "rxkad - Invalid ticket length."},
     {RXKADOUTOFSEQUENCE,      "rxkad - Out of sequence."},
     {RXKADNOAUTH,             "rxkad - No authentication."},
     {RXKADBADKEY,             "rxkad - Bad key."},
@@ -273,6 +274,17 @@ static struct koerr koerrmsg[] = {
     {KATOOSOON,			"ka - password changed too soon"},
     {KALOCKED,			"ka - account locked"},
 
+    /* rx errors */
+
+    {ARLA_CALL_DEAD,		"rx - call dead"},
+    {ARLA_INVALID_OPERATION,	"rx - invalid operation"},
+    {ARLA_CALL_TIMEOUT,		"rx - call timeout"},
+    {ARLA_EOF,			"rx - end of data"},
+    {ARLA_PROTOCOL_ERROR,	"rx - protocol error"},
+    {ARLA_USER_ABORT,		"rx - user abort"},
+    {ARLA_ADDRINUSE,		"rx - address already in use"},
+    {ARLA_MSGSIZE,		"rx - packet too big"},
+
     /* Not a known error */
 
     { 0L,              "Unknown error"}
