@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: k_getpwnam.c,v 1.1 1998/01/13 16:25:26 lha Exp $");
+RCSID("$Id: k_getpwnam.c,v 1.4 2002/09/12 16:22:35 lha Exp $");
 #endif /* HAVE_CONFIG_H */
 
 #include "roken.h"
@@ -47,12 +42,12 @@ RCSID("$Id: k_getpwnam.c,v 1.1 1998/01/13 16:25:26 lha Exp $");
 #endif
 
 struct passwd *
-k_getpwnam (char *user)
+k_getpwnam (const char *user)
 {
      struct passwd *p;
 
      p = getpwnam (user);
-#ifdef HAVE_GETSPNAM
+#if defined(HAVE_GETSPNAM) && defined(HAVE_STRUCT_SPWD)
      if(p)
      {
 	  struct spwd *spwd;
