@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2000, 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID ("$Id: rtbl.c,v 1.1 2000/08/16 01:23:45 lha Exp $");
+RCSID ("$Id: rtbl.c,v 1.3 2002/09/12 16:23:05 lha Exp $");
 #endif
 #include "roken.h"
 #include "rtbl.h"
@@ -83,12 +83,14 @@ rtbl_destroy (rtbl_t table)
 
 	for (j = 0; j < c->num_rows; j++)
 	    free (c->rows[j].data);
+	free (c->rows);
 	free (c->header);
 	free (c->prefix);
 	free (c);
     }
     free (table->column_prefix);
     free (table->columns);
+    free (table);
 }
 
 int
