@@ -1,23 +1,23 @@
 /*
- * Copyright (c) 1995 - 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 
+ *
  * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -36,7 +36,7 @@
 #endif
 #include "roken.h"
 
-RCSID("$Id: strlcpy.c,v 1.1 2000/08/16 01:23:48 lha Exp $");
+RCSID("$Id: strlcpy.c,v 1.3 2002/09/12 16:23:24 lha Exp $");
 
 #ifndef HAVE_STRLCPY
 
@@ -44,17 +44,17 @@ size_t
 strlcpy (char *dst, const char *src, size_t dst_sz)
 {
     size_t n;
-    char *p;
 
-    for (p = dst, n = 0;
-	 n + 1 < dst_sz && *src != '\0';
-	 ++p, ++src, ++n)
-	*p = *src;
-    *p = '\0';
-    if (*src == '\0')
+    for (n = 0; n < dst_sz; n++) {
+	if ((*dst++ = *src++) == '\0')
+	    break;
+    }
+
+    if (n < dst_sz)
 	return n;
-    else
-	return n + strlen (src);
+    if (n > 0)
+	*(dst - 1) = '\0';
+    return n + strlen (src);
 }
 
 #endif
