@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan 
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan 
  * (Royal Institute of Technology, Stockholm, Sweden).  
  * All rights reserved.
  * 
@@ -33,49 +33,20 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: warnerr.c,v 1.2.2.2 2001/07/16 11:59:47 assar Exp $");
+RCSID("$Id: warnerr.c,v 1.5 2002/09/12 16:23:50 lha Exp $");
 #endif
 
 #include "roken.h"
 #include "err.h"
 
-#ifndef HAVE___PROGNAME
-const char *__progname;
-#endif
-
-#ifndef HAVE_GET_PROGNAME
-const char *
-get_progname(void)
-{
-    return __progname;
-}
-#endif
-
-#ifndef HAVE_SET_PROGNAME
-void
-set_progname(char *argv0)
-{
-#ifndef HAVE___PROGNAME
-    char *p;
-    if(argv0 == NULL)
-	return;
-    p = strrchr(argv0, '/');
-    if(p == NULL)
-	p = argv0;
-    else
-	p++;
-    __progname = p;
-#endif
-}
-#endif
-
-#ifndef HAVE_WARNERR
 void
 warnerr(int doerrno, const char *fmt, va_list ap)
 {
     int sverrno = errno;
-    if(__progname != NULL){
-	fprintf(stderr, "%s", __progname);
+    const char *progname = getprogname();
+
+    if(progname != NULL){
+	fprintf(stderr, "%s", progname);
 	if(fmt != NULL || doerrno)
 	    fprintf(stderr, ": ");
     }
@@ -88,4 +59,3 @@ warnerr(int doerrno, const char *fmt, va_list ap)
 	fprintf(stderr, "%s", strerror(sverrno));
     fprintf(stderr, "\n");
 }
-#endif
