@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -43,7 +38,7 @@
  * multiple commands on the command line to run several tests at the
  * same time.
  * 
- * $Id: testlwp.c,v 1.3 1998/10/25 19:37:33 joda Exp $
+ * $Id: testlwp.c,v 1.9 2002/06/01 17:47:49 lha Exp $
  *
  */
 
@@ -52,13 +47,18 @@
 #include <string.h>
 #include <err.h>
 #include <lwp.h>
+#include <lock.h>
 #include <unistd.h>
 #include <sys/time.h>
 
 static void Producer(void *foo) ;
 static void Consumer(void *foo) ;
 
+#ifndef AFS_LWP_MINSTACKSIZE
 #define LWPTEST_STACKSIZE (16*1024)
+#else
+#define LWPTEST_STACKSIZE AFS_LWP_MINSTACKSIZE
+#endif
 
 /*
  * Classic Producer Consumer thread testing
@@ -164,6 +164,9 @@ SelectConsumer(void *foo)
     
     while(1) {
 	FD_ZERO(&readset);
+	if (pipa[0] >= FD_SETSIZE)
+	    errx (1, "fd too large");
+
 	FD_SET(pipa[0], &readset);
 	IOMGR_Select(pipa[0] + 1, &readset, NULL, NULL, NULL); 
 	len = read(pipa[0], str, 199);
@@ -174,7 +177,7 @@ SelectConsumer(void *foo)
     }
 }
 
-void
+static void
 startSelectPC (char *progname)
 {
     int pid;
@@ -245,6 +248,78 @@ yaEndlessLoop(void)
 	errx (1, "Cannot create enterprise process");
 }
 
+static void
+deadlock_write (void)
+{
+    struct Lock lock;
+    
+    Lock_Init (&lock);
+    ObtainWriteLock(&lock);
+    ObtainWriteLock(&lock);
+}
+
+static void
+deadlock_read (void)
+{
+    struct Lock lock;
+    
+    Lock_Init (&lock);
+    ObtainWriteLock(&lock);
+    ObtainReadLock(&lock);
+}
+
+static void
+deadlock_read2 (void)
+{
+    struct Lock lock;
+    
+    Lock_Init (&lock);
+    ObtainReadLock(&lock);
+    ObtainWriteLock(&lock);
+}
+
+/* 
+ * the terms overrun and underrun stack are used somewhat wrong here,
+ * in what direction do your stack grow
+ */
+
+/* 
+ * when testing if the stack thingy work, it might not be so smart to
+ * use the stack (automatic variables) for storage :)
+ */
+
+int stack_i, stack_printed = 0;
+
+static void
+overrun_stack (void *arg)
+{
+    char foo[10];
+    int stack_i, stack_printed = 0;
+
+    for (stack_i = 10; stack_i > -LWPTEST_STACKSIZE * 2; stack_i--) {
+	if (stack_i < -LWPTEST_STACKSIZE - 100 && !stack_printed) {
+	    printf("hum overrun stack now\n");
+	    stack_printed = 1;
+	}
+	foo[stack_i] = 0x4e;
+    }
+}
+
+
+static void
+underrun_stack (void *arg)
+{
+    char foo[10];
+
+    for (stack_i = 0; stack_i < LWPTEST_STACKSIZE * 2; stack_i++) {
+	if (stack_i > LWPTEST_STACKSIZE + 100&& !stack_printed) {
+	    printf("hum underrun stack now\n");
+	    stack_printed = 1;
+	}
+	foo[stack_i] = 0xe4;
+    }
+}
+
 
 /*
  * Usage
@@ -260,6 +335,11 @@ usage(char *progname)
 	    "selectconsumer\tSelect consumer\n"
 	    "selectproducer\t(special case, just print a string on stdout repeatally)\n"
 	    "cancel\t\tTest iomgr cancel\n"
+	    "deadlock-write\tdeadlockdetection\n"
+	    "deadlock-read\tdeadlockdetection\n"
+	    "deadlock-read2\tdeadlockdetection\n"
+	    "overrun-stack\tover run the stack\n"
+	    "underrun-stack\tunder run the stack\n"
 	    "version\t\tPrint version\n");
 
     printf("Use several of these tests together to test their interopability\n");
@@ -297,10 +377,29 @@ int main(int argc, char **argv)
 	    startSelectPC (progname);
 	} else if (strcasecmp("cancel", argv[1]) == 0) {
 	    yaEndlessLoop();	 
+	} else if (strcasecmp("deadlock-write", argv[1]) == 0) {
+	    deadlock_write();
+	} else if (strcasecmp("deadlock-read", argv[1]) == 0) {
+	    deadlock_read();
+	} else if (strcasecmp("deadlock-read2", argv[1]) == 0) {
+	    deadlock_read2();
+	} else if (strcasecmp("overrun-stack", argv[1]) == 0) {
+	    PROCESS tpid;
+	    if (LWP_CreateProcess (overrun_stack, LWPTEST_STACKSIZE, 1,
+				   NULL, "overunner", &tpid))
+		errx (1, "Cannot create stack overrunner process");
+	} else if (strcasecmp("underrun-stack", argv[1]) == 0) {
+	    PROCESS tpid;
+	    if (LWP_CreateProcess (underrun_stack, LWPTEST_STACKSIZE, 1,
+				   NULL, "underrunner", &tpid))
+		errx (1, "Cannot create stack underrunner process");
 	} else if (strcasecmp("version", argv[1]) == 0) {
-	    printf("Version: $Id: testlwp.c,v 1.3 1998/10/25 19:37:33 joda Exp $\n");
+	    printf("Version: "
+		   "$Id: testlwp.c,v 1.9 2002/06/01 17:47:49 lha Exp $\n");
+	    exit (0);
 	} else {
 	    printf("unknown command %s\n", argv[1]);
+	    exit (1);
 	}
 
 	argc--;
