@@ -33,7 +33,7 @@
 
 #include "fsrv_locl.h"
 
-RCSID("$Id: fileserver.c,v 1.25.2.1 2002/07/11 14:00:49 lha Exp $");
+RCSID("$Id: fileserver.c,v 1.30 2002/04/26 16:11:42 lha Exp $");
 
 typedef enum { NO_SALVAGE, NORMAL_SALVAGE, SALVAGE_ALL } salvage_options_e;
 
@@ -44,15 +44,6 @@ typedef enum { NO_SALVAGE, NORMAL_SALVAGE, SALVAGE_ALL } salvage_options_e;
 static struct rx_service *fsservice;
 static struct rx_service *volservice;
 
-static char *cell = NULL;
-static char *realm = NULL;
-static char *srvtab_file = NULL;
-static char *debug_levels = NULL;
-static char *log_file  = "syslog";
-static int no_auth = 0;
-static int force_salvage = 0;
-static int salvage_options = NORMAL_SALVAGE;
-
 /*
  *
  */
@@ -100,8 +91,10 @@ do_salvage (volume_handle *vol)
 static int
 salvage_and_attach (volume_handle *vol, void *arg)
 {
+    int salvage_options = *(int *)arg;
+
     mlog_log (MDEBMISC, "salvaging and attatching to volume: %u", 
-	      (u_int32_t)vol->vol);
+	      (uint32_t)vol->vol);
 
     switch (salvage_options) {
     case NORMAL_SALVAGE:
@@ -126,10 +119,10 @@ salvage_and_attach (volume_handle *vol, void *arg)
  */
 
 static void
-attach_volumes(void)
+attach_volumes(int salvage_options)
 {
     mlog_log (MDEBMISC, "fileserver starting to attach to volumes");
-    vld_iter_vol (salvage_and_attach, NULL);
+    vld_iter_vol (salvage_and_attach, &salvage_options);
     mlog_log (MDEBMISC, "fileserver done attaching to volumes");
 }
 
@@ -137,23 +130,34 @@ attach_volumes(void)
  * Main
  */
 
+static char *cell = NULL;
+static char *realm = NULL;
+static char *debug_levels = NULL;
+static char *log_file  = "syslog";
+static char *srvtab_file = NULL;
+static int no_auth = 0;
+static int do_help = 0;
+static int force_salvage = 0;
+
 static struct agetargs args[] = {
     {"cell",	0, aarg_string,    &cell, "what cell to use"},
     {"realm",	0, aarg_string,	  &realm, "what realm to use"},
-    {"noauth",	0, aarg_flag,	  &no_auth, "disable authentication checks"},
-    {"debug",	0, aarg_string,	  &debug_levels, "debug level"},
+    {"debug",  'd', aarg_string,  &debug_levels, "debug level"},
     {"log",	'l',	aarg_string,	&log_file,
-     "where to write log (stderr (default), syslog, or path to file)"},
-    {"srvtab",'s', aarg_string,    &srvtab_file, "what srvtab to use"},
+     "where to write log (stderr, syslog (default), or path to file)"},
+    {"srvtab",  0, aarg_string,    &srvtab_file, "what srvtab to use"},
+    {"noauth",	0, aarg_flag,	  &no_auth, "disable authentication checks"},
+    {"help",  'h', aarg_flag,      &do_help, "help"},
+    {"partdir", 0, aarg_string,    &dpart_root, "where to find vicep*"},
     {"salvage", 0, aarg_flag,      &force_salvage, "Force a salvage for all vols"},
-    {"partdir",'s', aarg_string,    &dpart_root, "where to find vicep*"},
     { NULL, 0, aarg_end, NULL }
 };
 
 static void
-usage(void)
+usage(int exit_code)
 {
-    aarg_printusage(args, "fileserver", "", AARG_GNUSTYLE);
+    aarg_printusage (args, NULL, "", AARG_GNUSTYLE);
+    exit (exit_code);
 }
 
 int 
@@ -163,12 +167,12 @@ main(int argc, char **argv)
     int optind = 0;
     PROCESS pid;
     Log_method *method;
+    int salvage_options = NORMAL_SALVAGE;
     
-    setprogname (argv[0]);
+    set_progname (argv[0]);
 
     if (agetarg (args, argc, argv, &optind, AARG_GNUSTYLE)) {
-	usage ();
-	return 1;
+	usage (1);
     }
 
     argc -= optind;
@@ -179,6 +183,9 @@ main(int argc, char **argv)
 	return 1;
     }
 
+    if (do_help)
+	usage(0);
+
     method = log_open (getprogname(), log_file);
     if (method == NULL)
 	errx (1, "log_open failed");
@@ -193,17 +200,17 @@ main(int argc, char **argv)
     if (debug_levels)
 	mlog_log_set_level (debug_levels);
     
-    if (force_salvage)
-      salvage_options = SALVAGE_ALL;
-
-    ropa_init(30000, 100, 40000, 40000, 150, 50000);
-
     if (no_auth)
 	sec_disable_superuser_check ();
 
     if (cell)
 	cell_setthiscell (cell);
 
+    if (force_salvage)
+	salvage_options = SALVAGE_ALL;
+
+    ropa_init(30000, 100, 40000, 40000, 150, 50000);
+
     network_kerberos_init (srvtab_file);
     
     ret = network_init(htons(afsport), "fs", FS_SERVICE_ID, 
@@ -225,7 +232,7 @@ main(int argc, char **argv)
     vld_init();
 
     mnode_init (4711); /* XXX */
-    attach_volumes();
+    attach_volumes(salvage_options);
     
     rx_SetMaxProcs(fsservice,5) ;
     rx_SetMaxProcs(volservice,5) ;
