@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,19 @@
 
 #include "fsrv_locl.h"
 
-RCSID("$Id: volprocs.c,v 1.9 2000/10/03 00:17:36 lha Exp $");
+RCSID("$Id: volprocs.c,v 1.22 2003/11/14 09:33:42 tol Exp $");
+
+/*
+ * exit debug logging
+ */
+
+#ifndef HAVE___FUNCTION
+#define __FUNCTION__ "unknown-function"
+#endif
+
+#define VOLSER_EXIT(r)							\
+	mlog_log(MDEBVOLDB, 						\
+		 "%s error: %s(%d)", __FUNCTION__, koerr_gettext(r), r)
 
 /*
  * Helper function
@@ -48,14 +60,16 @@ volser_fetch_vh (int32_t partition, int32_t volid,
     ret = dp_create (partition, dp);
     if (ret) {
 	fprintf (stderr, "volser_fetch_vh: dp_create: %d\n", ret);
-	return VOLSERFAILEDOP; /* XXX */
+	return VOLSERILLEGAL_PARTITION;
     }
 
     ret = vld_open_volume_by_num (*dp, volid, vh);
     if (ret) {
 	fprintf (stderr, "volser_fetch_vh: vld_open_volume_by_num: %d\n", ret);
 	dp_free (*dp);
-	return VOLSERFAILEDOP; /* XXX */
+	if (ret == ENOENT)
+	    return VNOVOL;
+	return ret;
     }
     
     ret = vld_info_uptodatep (*vh);
@@ -73,7 +87,7 @@ volser_fetch_vh (int32_t partition, int32_t volid,
  */
 
 int
-VOLSER_AFSVolCreateVolume(struct rx_call *call,
+SVOLSER_AFSVolCreateVolume(struct rx_call *call,
 			  const int32_t partition,
 			  const char *name,
 			  const int32_t type,
@@ -81,38 +95,46 @@ VOLSER_AFSVolCreateVolume(struct rx_call *call,
 			  int32_t *volid,
 			  int32_t *trans)
 {
-    int ret;
+    int ret = 0;
     int32_t backstoretype = VLD_SVOL;
     struct dp_part *dp;
 
-    mlog_log(MDEBVOLDB, "VOLSER_AFSVolCreateVolume "
-	     "part %d name %s type %d parent %d volid %u",
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolCreateVolume"
+	     " part %d name %s type %d parent %d volid %u",
 	     partition, name, type, parent, *volid);
 
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
     /* XXX parent should be used */
 
     ret = vld_create_trans (partition, *volid, trans);
 
     if (ret)
-	return ret;
+	goto out;
 
-    ret = vld_trans_set_iflags (*trans, ITOffline);
+    ret = vld_trans_set_iflags (*trans, ITCreate);
     if (ret)
-	return ret;
+	goto out;
 
     ret = dp_create (partition, &dp);
     if (ret) {
 	vld_end_trans (*trans, NULL);
-	return ret;
+	goto out;
     }
 
     ret = vld_create_volume (dp, *volid, name, backstoretype, type, 0);
     if (ret) {
 	vld_end_trans (*trans, NULL);
-	return ret;
+	goto out;
     }
 
-    return 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -120,16 +142,43 @@ VOLSER_AFSVolCreateVolume(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolDeleteVolume(struct rx_call *call,
-			  const int32_t trans)
+SVOLSER_AFSVolDeleteVolume(struct rx_call *call,
+			  const int32_t transid)
 {
-    int ret;
+    int ret = 0;
+    int32_t backstoretype = VLD_SVOL;
+    struct trans *trans;
+    struct dp_part *dp;
 
-    ret = vld_verify_trans(trans);
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolDeleteVolume");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = vld_verify_trans(transid);
     if (ret)
-	return ret;
+	goto out;
 
-    return VOLSERFAILEDOP;
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	goto out;
+
+    ret = dp_create (trans->partition, &dp);
+    if (ret) {
+	vld_put_trans (trans);
+	goto out;
+    }
+
+    ret = vld_delete_volume (dp, trans->volid, backstoretype, 0);
+
+    vld_put_trans(trans);
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -137,11 +186,25 @@ VOLSER_AFSVolDeleteVolume(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolNukeVolume(struct rx_call *call,
+SVOLSER_AFSVolNukeVolume(struct rx_call *call,
 			const int32_t partID,
 			const int32_t volID)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolNukeVolume");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -149,42 +212,52 @@ VOLSER_AFSVolNukeVolume(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolDump(struct rx_call *call,
+SVOLSER_AFSVolDump(struct rx_call *call,
 		  const int32_t fromTrans,
 		  const int32_t fromDate)
 {
-    int ret;
+    int ret = 0;
     struct trans *trans;
     struct dp_part *dp;
     volume_handle *vh;
 
     mlog_log (MDEBVOLDB, 
-	      "VOLSER_AFSVolDump trans %d fromdate %d", 
+	      "VOLSER_AFSVolDump: trans %d fromdate %d", 
 	      fromTrans, fromDate);
     
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
     ret = vld_verify_trans (fromTrans);
     if (ret)
-	return ret;
+	goto out;
 
     ret = vld_get_trans (fromTrans, &trans);
     if (ret)
-	return ret;
+	goto out;
     
     ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
     if (ret) {
 	vld_put_trans(trans);
-	return ret;
+	goto out;
     }
 
     if (fromDate != 0) {
 	vld_put_trans(trans);
-	return VOLSERFAILEDOP;
+	ret = VOLSERFAILEDOP;
+	goto out;
     }
 
-
+    ret = generate_dump(call, vh);
 
     vld_put_trans (trans);
-    return VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -192,13 +265,27 @@ VOLSER_AFSVolDump(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolSignalRestore(struct rx_call *call,
+SVOLSER_AFSVolSignalRestore(struct rx_call *call,
 			   const char *name,
 			   const int32_t type,
 			   const int32_t pid,
 			   const int32_t cloneid)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSignalRestore");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -206,12 +293,58 @@ VOLSER_AFSVolSignalRestore(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolRestore(struct rx_call *call,
-		     const int32_t toTrans,
+SVOLSER_AFSVolRestore(struct rx_call *call,
+		     const int32_t transid,
 		     const int32_t flags,
 		     const struct restoreCookie *cookie)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+    struct trans *trans;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolRestore");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = vld_verify_trans(transid);
+    if (ret)
+	goto out;
+
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	goto out;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	goto out;
+    }
+
+    ret = parse_dump(call, vh);
+    if (ret) {
+	vld_free (vh);
+	dp_free (dp);
+	vld_put_trans(trans);
+	goto out;
+    }
+
+    ret = vld_rebuild(vh);
+
+    vld_info_write(vh);
+
+    vld_free (vh);
+    dp_free (dp);
+
+    vld_put_trans(trans);
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -219,14 +352,28 @@ VOLSER_AFSVolRestore(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolForward(struct rx_call *call,
+SVOLSER_AFSVolForward(struct rx_call *call,
 		     const int32_t fromTrans,
 		     const int32_t fromData,
 		     const struct destServer *destination,
 		     const int32_t destTrans,
 		     const struct restoreCookie *cookie)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolForward");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -234,14 +381,30 @@ VOLSER_AFSVolForward(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolClone(struct rx_call *call,
+SVOLSER_AFSVolClone(struct rx_call *call,
 		   const int32_t trans,
 		   const int32_t purgeVol,
 		   const int32_t newType,
 		   const char *newName,
 		   int32_t *newVol)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolClone"
+	     " trans %d purgevol %d newtype %d newname %d",
+	     trans, purgeVol, newType, newName);
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -249,11 +412,25 @@ VOLSER_AFSVolClone(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolReClone(struct rx_call *call,
+SVOLSER_AFSVolReClone(struct rx_call *call,
 		     const int32_t tid,
 		     const int32_t cloneID)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolReClone");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -261,11 +438,25 @@ VOLSER_AFSVolReClone(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolSetForwarding(struct rx_call *call,
+SVOLSER_AFSVolSetForwarding(struct rx_call *call,
 			   const int32_t tid,
 			   const int32_t newsite)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetForwarding");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -273,25 +464,44 @@ VOLSER_AFSVolSetForwarding(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolTransCreate(struct rx_call *call,
+SVOLSER_AFSVolTransCreate(struct rx_call *call,
 			 const int32_t volume,
 			 const int32_t partition,
 			 const int32_t flags,
 			 int32_t *trans)
 {
-    int ret;
+    int ret = 0;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolTransCreate");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = volser_fetch_vh (partition, volume, &dp, &vh);
+    if (ret)
+	goto out;
+
+    vld_free (vh);
+    dp_free (dp);
 
     ret = vld_create_trans(partition, volume, trans);
     if (ret)
-	return ret;
+	goto out;
 
     ret = vld_trans_set_iflags(*trans, flags);
     if (ret) {
 	vld_end_trans (*trans, NULL);
-	return ret;
+	goto out;
     }
 
-    return 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -299,21 +509,40 @@ VOLSER_AFSVolTransCreate(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolEndTrans(struct rx_call *call,
-		      const int32_t trans,
+SVOLSER_AFSVolEndTrans(struct rx_call *call,
+		      const int32_t transid,
 		      int32_t *rcode)
 {
-    int ret;
+    struct trans *trans;
+    int ret = 0;
 
-    ret = vld_verify_trans(trans);
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolEndTrans trans %d", transid);
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = vld_verify_trans(transid);
+    if (ret) 
+	goto out;
+
+    ret = vld_get_trans(transid, &trans);
     if (ret)
-	return ret;
+	goto out;
 
-    ret = vld_end_trans(trans, rcode);
+    ropa_break_volume_callback(trans->volid); /* XXX */
+
+    vld_put_trans(trans);
+
+    ret = vld_end_trans(transid, rcode);
     if (ret)
-	return ret;
+	goto out;
 
-    return 0;
+ out:
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolEndTrans returns %d", ret);
+
+    return ret;
 }
 
 /*
@@ -321,21 +550,31 @@ VOLSER_AFSVolEndTrans(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolGetFlags(struct rx_call *call,
+SVOLSER_AFSVolGetFlags(struct rx_call *call,
 		      const int32_t trans,
 		      int32_t *flags)
 {
-    int ret;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolGetFlags");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
 
     ret = vld_verify_trans(trans);
     if (ret)
-	return ret;
+	goto out;
 
     ret = vld_trans_get_vflags(trans, flags);
     if (ret)
-	return ret;
+	goto out;
 
-    return 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -343,30 +582,35 @@ VOLSER_AFSVolGetFlags(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolSetFlags(struct rx_call *call,
+SVOLSER_AFSVolSetFlags(struct rx_call *call,
 		      const int32_t transid,
 		      const int32_t flags)
 {
-    int ret;
+    int ret = 0;
     struct trans *trans;
     struct dp_part *dp;
     volume_handle *vh;
 
-    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetFlags "
-	     "trans %d flags %d", transid, flags);
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetFlags"
+	     " trans %d flags %d", transid, flags);
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
 
     ret = vld_verify_trans(transid);
     if (ret)
-	return ret;
+	goto out;
 
     ret = vld_get_trans(transid, &trans);
     if (ret)
-	return ret;
+	goto out;
     
     ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
     if (ret) {
 	vld_put_trans(trans);
-	return ret;
+	goto out;
     }
 
     if (flags & VTDeleteOnSalvage)
@@ -374,7 +618,9 @@ VOLSER_AFSVolSetFlags(struct rx_call *call,
     else
 	vh->info.destroyMe = 0;
 
+#if 0
     assert ((flags & VTOutOfService) == 0); /* XXX */
+#endif
 
     vld_info_write(vh);
 
@@ -385,9 +631,12 @@ VOLSER_AFSVolSetFlags(struct rx_call *call,
 
     ret = vld_trans_set_vflags(transid, flags);
     if (ret)
-	return ret;
+	goto out;
 
-    return 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -395,11 +644,25 @@ VOLSER_AFSVolSetFlags(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolGetName(struct rx_call *call,
+SVOLSER_AFSVolGetName(struct rx_call *call,
 		     const int32_t tid,
 		     char tname[256])
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolGetName");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -407,11 +670,62 @@ VOLSER_AFSVolGetName(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolGetStatus(struct rx_call *call,
-		       const int32_t tid,
+SVOLSER_AFSVolGetStatus(struct rx_call *call,
+		       const int32_t transid,
 		       struct volser_status *status)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+    struct trans *trans;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolGetStatus");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = vld_verify_trans(transid);
+    if (ret)
+	goto out;
+
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	goto out;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	goto out;
+    }
+
+    status->volID = vh->info.volid;
+    status->nextUnique = 0 /* XXX */;
+    status->type = vh->info.type;
+    status->parentID = vh->info.parentID;
+    status->cloneID = vh->info.cloneID;
+    status->backupID = vh->info.backupID;
+    status->restoredFromID = 0 /* XXX */;
+    status->maxQuota = vh->info.maxquota;
+    status->minQuota = 0 /* XXX */;
+    status->owner = 0 /* XXX */;
+    status->creationDate = vh->info.creationDate;
+    status->accessDate = vh->info.accessDate;
+    status->updateDate = vh->info.updateDate;
+    status->exprirationDate = 0 /* XXX */;
+    status->backupDate = vh->info.backupDate;
+    status->copyDate = 0 /* XXX */;
+
+    vld_free (vh);
+    dp_free (dp);
+
+    vld_put_trans(trans);
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -419,15 +733,58 @@ VOLSER_AFSVolGetStatus(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolSetIdsTypes(struct rx_call *call,
-			 const int32_t tId,
+SVOLSER_AFSVolSetIdsTypes(struct rx_call *call,
+			 const int32_t transid,
 			 const char *name,
 			 const int32_t type,
-			 const int32_t pId,
-			 const int32_t cloneId,
-			 const int32_t backupId)
+			 const int32_t parentID,
+			 const int32_t cloneID,
+			 const int32_t backupID)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+    struct trans *trans;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetIdsTypes: type %d parentID %d "
+	     "cloneID %d backupID %d", type, parentID, cloneID, backupID);
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = vld_verify_trans(transid);
+    if (ret)
+	goto out;
+
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	goto out;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	goto out;
+    }
+
+    strlcpy(vh->info.name, name, VNAMESIZE);
+    vh->info.type = type;
+    vh->info.parentID = parentID;
+    vh->info.cloneID = cloneID;
+    vh->info.backupID = backupID;
+
+    vld_info_write(vh);
+
+    vld_free (vh);
+    dp_free (dp);
+
+    vld_put_trans(trans);
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -435,11 +792,49 @@ VOLSER_AFSVolSetIdsTypes(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolSetDate(struct rx_call *call,
-		     const int32_t tid,
+SVOLSER_AFSVolSetDate(struct rx_call *call,
+		     const int32_t transid,
 		     const int32_t newDate)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+    struct trans *trans;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetDate");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
+    ret = vld_verify_trans(transid);
+    if (ret)
+	goto out;
+
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	goto out;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	goto out;
+    }
+
+    vh->info.creationDate = newDate;
+
+    vld_info_write(vh);
+
+    vld_free (vh);
+    dp_free (dp);
+
+    vld_put_trans(trans);
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -447,12 +842,14 @@ VOLSER_AFSVolSetDate(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolListPartitions(struct rx_call *call,
+SVOLSER_AFSVolListPartitions(struct rx_call *call,
 			    struct pIDs *partIDs)
 {
     int i;
     struct dp_part *dp = NULL;
-    int ret;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolListPartitions");
 
     i = 0;
     do {
@@ -460,7 +857,7 @@ VOLSER_AFSVolListPartitions(struct rx_call *call,
 	if (dp == NULL)
 	    break;
 	if (ret)
-	    return ret;
+	    goto out;
 	partIDs->partIds[i] = dp->num;
 	i++;
     } while (i < 26);
@@ -468,7 +865,11 @@ VOLSER_AFSVolListPartitions(struct rx_call *call,
     for (; i < 26; i++)
 	partIDs->partIds[i] = -1;
     
-    return 0;
+    ret = 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -476,77 +877,51 @@ VOLSER_AFSVolListPartitions(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolPartitionInfo(struct rx_call *call,
+SVOLSER_AFSVolPartitionInfo(struct rx_call *call,
 			   const char *name,
 			   struct diskPartition *partition)
 {
     int num;
     struct dp_part *dp;
-    int ret;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolPartitionInfo");
 
     num = partition_name2num (name);
-    if (num == -1)
-	return 0; /* XXX */
+    if (num == -1) {
+	ret = VOLSERILLEGAL_PARTITION;
+	goto out;
+    }
     
     ret = dp_create (num, &dp);
     if (ret)
-	return ret;
+	goto out;
 
     memset(partition, 0, sizeof(*partition));
-    strlcpy(partition->name, dp->part, 32);    
+    strlcpy(partition->name, dp->part, 32);
+    partition->free = 1000;
     
     dp_free(dp);
+
+ out:
+    VOLSER_EXIT(ret);
     
-    return 0;
+    return ret;
 }
 
-/*
- *
- */
 
-int
-VOLSER_AFSVolListVolumes(struct rx_call *call,
-			 const int32_t partID,
-			 const int32_t flags,
-			 volEntries *resultEntries)
+static void
+copy_volumeinfo(struct volintInfo *volinfo,
+		volume_handle *vh,
+		const int32_t partID)
 {
-    return VOLSERFAILEDOP;
-}
-
-/*
- *
- */
-
-int
-VOLSER_AFSVolListOneVolume(struct rx_call *call,
-			   const int32_t partID,
-			   const int32_t volid,
-			   volEntries *resultEntries)
-{
-    volume_handle *vh;
-    int ret;
-    struct dp_part *dp;
-    struct volintInfo *volinfo;
-
-    fprintf(stderr, 
-	    "VOLSER_AFSVolListOneVolume partid: %d volid: %u\n",
-	    partID, volid);
-
-    ret = volser_fetch_vh (partID, volid, &dp, &vh);
-    if (ret)
-	return ret;
-
-    resultEntries->len = 1;
-    
-    volinfo = calloc(sizeof(struct volintInfo) * resultEntries->len, 1);
-    resultEntries->val = volinfo;
     strlcpy(volinfo->name, vh->info.name, VNAMESIZE);
     volinfo->volid = vh->info.volid;
     volinfo->type = vh->info.type;
     volinfo->backupID = vh->info.backupID;
     volinfo->parentID = vh->info.parentID;
     volinfo->cloneID = vh->info.cloneID;
-    if (vld_check_busy(volid, partID))
+    if (vld_check_busy(volinfo->volid, partID))
 	volinfo->status = VBUSY;
     else
 	volinfo->status = VOK;
@@ -562,12 +937,105 @@ VOLSER_AFSVolListOneVolume(struct rx_call *call,
     volinfo->size = vh->info.size;
     volinfo->needsSalvaged = 0;
     volinfo->destroyMe = 0;
+}
+
+/*
+ *
+ */
+
+int
+SVOLSER_AFSVolListVolumes(struct rx_call *call,
+			 const int32_t partID,
+			 const int32_t flags,
+			 volEntries *resultEntries)
+{
+    int ret = 0;
+    List *vollist;
+    Listitem *item;
+    volume_handle *vh;
+    struct dp_part *dp;
+    int numvol;
+    int i;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolListVolumes");
+
+    ret = dp_create (partID, &dp);
+    if (ret)
+	goto out;
+
+    ret = vld_list_volumes(dp, &vollist);
+    if (ret)
+	goto free_part;
+
+    numvol = 0;
+
+    item = listhead(vollist);
+    while (item) {
+	numvol++;
+	item = listnext(vollist, item);
+    }
+
+    resultEntries->len = numvol;
+    
+    resultEntries->val = calloc(sizeof(struct volintInfo) * resultEntries->len, 1);
+
+    i = 0;
+    while (!listemptyp(vollist)) {
+	vh = (volume_handle *) listdelhead(vollist);
+	assert(vh);
+	ret = vld_info_uptodatep (vh);
+	assert(ret == 0);
+	copy_volumeinfo(&resultEntries->val[i], vh, partID);
+	vld_free (vh);
+	i++;
+    }
+
+    free(vollist);
+
+ free_part:
+    dp_free (dp);
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
+}
+
+/*
+ *
+ */
+
+int
+SVOLSER_AFSVolListOneVolume(struct rx_call *call,
+			   const int32_t partID,
+			   const int32_t volid,
+			   volEntries *resultEntries)
+{
+    volume_handle *vh;
+    int ret = 0;
+    struct dp_part *dp;
+    struct volintInfo *volinfo;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolListOneVolume partid: %d volid: %u\n",
+	     partID, volid);
+
+    ret = volser_fetch_vh (partID, volid, &dp, &vh);
+    if (ret)
+	goto out;
+
+    resultEntries->len = 1;
+    
+    volinfo = calloc(sizeof(struct volintInfo) * resultEntries->len, 1);
+    resultEntries->val = volinfo;
+    copy_volumeinfo(volinfo, vh, partID);
 
     vld_free (vh);
     dp_free (dp);
 
-
-    return 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -575,12 +1043,20 @@ VOLSER_AFSVolListOneVolume(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolGetNthVolume(struct rx_call *call,
+SVOLSER_AFSVolGetNthVolume(struct rx_call *call,
 			  const int32_t index,
 			  int32_t *volume,
 			  int32_t *partition)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolGetNthVolume");
+
+    ret = VOLSERFAILEDOP;
+
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -588,10 +1064,18 @@ VOLSER_AFSVolGetNthVolume(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolMonitor(struct rx_call *call,
+SVOLSER_AFSVolMonitor(struct rx_call *call,
 		     transDebugEntries *result)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolMonitor");
+
+    ret = VOLSERFAILEDOP;
+
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -599,12 +1083,20 @@ VOLSER_AFSVolMonitor(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolXListVolumes(struct rx_call *call,
+SVOLSER_AFSVolXListVolumes(struct rx_call *call,
 			  const int32_t partID,
 			  const int32_t flags,
 			  xvolEntries *resultEntries)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolXListVolumes");
+
+    ret = VOLSERFAILEDOP;
+
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -612,19 +1104,18 @@ VOLSER_AFSVolXListVolumes(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolXListOneVolume(struct rx_call *call,
+SVOLSER_AFSVolXListOneVolume(struct rx_call *call,
 			    const int32_t partID,
 			    const int32_t volid,
 			    xvolEntries *resultEntries)
 {
     volume_handle *vh;
-    int ret;
     struct dp_part *dp;
     struct xvolintInfo *volinfo;
+    int ret = 0;
 
-    fprintf(stderr, 
-	    "VOLSER_AFSVolXListOneVolume partid: %d volid: %u\n",
-	    partID, volid);
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolXListOneVolume partid: %d volid: %u\n",
+	     partID, volid);
 
     ret = volser_fetch_vh (partID, volid, &dp, &vh);
     if (ret)
@@ -658,8 +1149,9 @@ VOLSER_AFSVolXListOneVolume(struct rx_call *call,
     vld_free (vh);
     dp_free (dp);
 
-
-    return 0;
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -667,32 +1159,37 @@ VOLSER_AFSVolXListOneVolume(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolSetInfo(struct rx_call *call,
+SVOLSER_AFSVolSetInfo(struct rx_call *call,
 		     const int32_t transid,
 		     const struct volintInfo *volinfo)
 {
     volume_handle *vh;
-    int ret;
+    int ret = 0;
     struct dp_part *dp;
     struct trans *trans;
 
-    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetInfo "
-	     "trans %d name %s type %d parent %d volid %d backup %d",
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetInfo"
+	     " trans %d name %s type %d parent %d volid %d backup %d",
 	     transid, volinfo->name, volinfo->type, volinfo->parentID,
 	     volinfo->volid, volinfo->backupID);
 
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+
     ret = vld_verify_trans(transid);
     if (ret)
-	return ret;
+	goto out;
 
     ret = vld_get_trans(transid, &trans);
     if (ret)
-	return ret;
+	goto out;
     
     ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
     if (ret) {
 	vld_put_trans(trans);
-	return ret;
+	goto out;
     }
 
     if (volinfo->name[0])
@@ -733,7 +1230,10 @@ VOLSER_AFSVolSetInfo(struct rx_call *call,
     dp_free (dp);
     vld_put_trans(trans);
 
-    return 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -741,12 +1241,12 @@ VOLSER_AFSVolSetInfo(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolXListPartitions(struct rx_call *call,
+SVOLSER_AFSVolXListPartitions(struct rx_call *call,
 			     part_entries *ent)
 {
     int i;
     struct dp_part *dp = NULL;
-    int ret;
+    int ret = 0;
     int32_t partIDs[26]; /* XXX */
 
     mlog_log(MDEBVOLDB, "VOLSER_AFSVolXListPartitions");
@@ -757,7 +1257,7 @@ VOLSER_AFSVolXListPartitions(struct rx_call *call,
 	if (dp == NULL)
 	    break;
 	if (ret)
-	    return ret;
+	    goto out;
 	partIDs[i] = dp->num;
 	i++;
     } while (i < 26);
@@ -766,7 +1266,11 @@ VOLSER_AFSVolXListPartitions(struct rx_call *call,
     ent->val = malloc(sizeof(int32_t) * ent->len);
     memcpy(ent->val, partIDs, sizeof(int32_t) * ent->len);
 
-    return 0;
+    ret = 0;
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 /*
@@ -774,15 +1278,29 @@ VOLSER_AFSVolXListPartitions(struct rx_call *call,
  */
 
 int
-VOLSER_AFSVolForwardMultiple(struct rx_call *call,
+SVOLSER_AFSVolForwardMultiple(struct rx_call *call,
 			     const int32_t fromTrans,
 			     const int32_t fromData,
-			     const replicas *destinations,
+			     const manyDests *destinations,
 			     const int32_t spare0,
 			     const struct restoreCookie *cookie,
-			     const multi_results *results)
+			     multi_results *results)
 {
-    return VOLSERFAILEDOP;
+    int ret = 0;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolForwardMultiple");
+
+    if (!sec_is_superuser(call)) {
+	ret = VOLSERBAD_ACCESS;
+	goto out;
+    }
+    
+    ret = VOLSERFAILEDOP;
+
+ out:
+    VOLSER_EXIT(ret);
+    
+    return ret;
 }
 
 
