@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: netinit.h,v 1.2 1999/06/09 22:43:02 map Exp $ */
+/* $Id: netinit.h,v 1.8 2000/10/03 00:18:24 lha Exp $ */
 
 void network_kerberos_init (char *srvtab);
 
@@ -44,6 +39,11 @@ int
 network_init (int serviceport, 
 	      char *servicename, 
 	      int serviceid,
-	      int (*request) (struct rx_call *),
+	      int32_t (*request) (struct rx_call *),
 	      struct rx_service **service,
 	      const char *realm);
+
+struct rx_securityClass *
+netinit_client_getcred(void);
+char *netinit_getrealm(void);
+
