@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,32 +33,7 @@
 
 #include "vldb_locl.h"
 
-RCSID("$Id: vldbserver.c,v 1.33.2.1 2002/07/11 14:00:51 lha Exp $");
-
-static void make_vldb_from_vl(struct vldbentry *vldb_entry,
-			      struct disk_vlentry *vl_entry);
-
-static void
-make_vldb_from_vl(struct vldbentry *vldb_entry, struct disk_vlentry *vl_entry)
-{
-    int i;
-
-    strlcpy(vldb_entry->name, vl_entry->name, VLDB_MAXNAMELEN);
-    vldb_entry->volumeType = vl_entry->volumeType;
-    vldb_entry->nServers = 1; /* XXX What is this supposed to be? */
-
-    for (i = 0; i < MAXNSERVERS; i++) {
-	vldb_entry->serverNumber[i] = vl_entry->serverNumber[i];
-	vldb_entry->serverPartition[i] = vl_entry->serverPartition[i];
-	vldb_entry->serverFlags[i] = vl_entry->serverFlags[i];
-    }
-
-    for (i = 0; i < MAXTYPES; i++)
-	vldb_entry->volumeId[i] = vl_entry->volumeId[i];
-
-    vldb_entry->cloneId = vl_entry->cloneId;
-    vldb_entry->flags = vl_entry->flags;
-}
+RCSID("$Id: vldbserver.c,v 1.46 2003/02/15 16:03:35 map Exp $");
 
 /*
  * The rpc - calls
@@ -68,42 +43,55 @@ int
 VL_CreateEntry(struct rx_call *call, 
 	       const vldbentry *newentry) 
 {
-    struct disk_vlentry vl_entry;
-    int32_t nServers;
-    int i;
+    char *name;
+    disk_vlentry diskentry;
+    disk_vlentry tempentry;
+
+    mlog_log (MDEBVL, "VL_CreateEntry (name=%s, ids=%d,%d,%d flags=%d)\n",
+	      newentry->name,
+	      newentry->volumeId[RWVOL],
+	      newentry->volumeId[ROVOL],
+	      newentry->volumeId[BACKVOL],
+	      newentry->flags);
+
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
-    memset (&vl_entry, 0, sizeof(vl_entry));
+    if ((vldb_id_to_name(newentry->volumeId[RWVOL], &name) == 0) ||
+	(vldb_id_to_name(newentry->volumeId[ROVOL], &name) == 0) ||
+	(vldb_id_to_name(newentry->volumeId[BACKVOL], &name) == 0)) {
+	free(name);
+	mlog_log (MDEBVL, "VL_CreateEntry: id exists\n");
+	return VL_NAMEEXIST;
+    }
 
-    vldb_debug ("VL_CreateEntry (name=%s, type=%d, ids=%d,%d,%d flags=%d)\n",
-		newentry->name, newentry->volumeType,
-		newentry->volumeId[RWVOL],
-		newentry->volumeId[ROVOL],
-		newentry->volumeId[BACKVOL],
-		newentry->flags);
+    if (vldb_read_entry(newentry->name, &tempentry) == 0) {
+	mlog_log (MDEBVL, "VL_CreateEntry: name exists\n");
+	return VL_NAMEEXIST;
+    }
 
-    strlcpy(vl_entry.name, newentry->name, VLDB_MAXNAMELEN);
-    vl_entry.volumeType = newentry->volumeType;
+    vldb_entry_to_disk(newentry, &diskentry);
 
-    /* XXX All fields mustn't be set */
-    nServers = newentry->nServers;
-    if (nServers > MAXNSERVERS)
-	nServers = MAXNSERVERS;
-    for (i = nServers - 1 ; i >= 0 ; i--) {
-	vl_entry.serverNumber[i] = newentry->serverNumber[i];
-	vl_entry.serverPartition[i] = newentry->serverPartition[i];
-	vl_entry.serverFlags[i] = newentry->serverFlags[i];
-    }    
+    if (vldb_write_entry(&diskentry) != 0)
+	return VL_IO;
 
-    for (i = 0; i < MAXTYPES; i++)
-	vl_entry.volumeId[i] = newentry->volumeId[i];
+    if (vldb_write_id(newentry->name,
+		      newentry->volumeId[RWVOL]) != 0)
+	return VL_IO; /* XXX rollback */
 
-    vl_entry.cloneId = newentry->cloneId;
-    vl_entry.flags = newentry->flags;
+    if (vldb_write_id(newentry->name,
+		      newentry->volumeId[ROVOL]) != 0)
+	return VL_IO; /* XXX rollback */
+
+    if (vldb_write_id(newentry->name,
+		      newentry->volumeId[BACKVOL]) != 0)
+	return VL_IO; /* XXX rollback */
+
+    vldb_flush();
+
+    vldb_free_diskentry(&diskentry);
 
-    vldb_insert_entry(&vl_entry);
     return 0;
 }
 
@@ -112,12 +100,74 @@ VL_DeleteEntry(struct rx_call *call,
 	       const int32_t Volid, 
 	       const int32_t voltype)
 {
-    vldb_debug ("VL_DeleteEntry\n") ;
+    disk_vlentry entry;
+    char *name;
+    int ret;
 
-    if (!sec_is_superuser(call))
-	return VL_PERM;
+    mlog_log (MDEBVL, "VL_DeleteEntry (Volid=%d,Voltype=%d)\n", 
+		  Volid, voltype);
 
-    return VL_PERM ;
+    if (!sec_is_superuser(call)) {
+	ret =  VL_PERM;
+	goto out;
+    }
+    
+    if (voltype != RWVOL &&
+	voltype != ROVOL &&
+	voltype != BACKVOL) {
+	ret =  VL_BADVOLTYPE;
+	goto out;
+    }
+    
+    if (vldb_id_to_name(Volid, &name)) {
+	ret =  VL_NOENT;
+	goto out;
+    }
+    
+    if (vldb_read_entry(name, &entry) != 0) {
+	ret =  VL_NOENT;
+	goto out;
+    }
+    
+    if (entry.volumeId[voltype] != Volid) {
+	ret =  VL_NOENT;
+	goto out;
+    }
+    
+    if (vldb_delete_id(name, entry.volumeId[RWVOL])) {
+	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove RW id %d\n",
+		  entry.volumeId[RWVOL]);
+	ret =  VL_IO;
+	goto out;
+    }
+    if (vldb_delete_id(name, entry.volumeId[ROVOL])) {
+	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove RO id %d\n",
+		  entry.volumeId[ROVOL]);
+	ret =  VL_IO;
+	goto out;
+    }
+    if (vldb_delete_id(name, entry.volumeId[BACKVOL])) {
+	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove BK id %d\n",
+		  entry.volumeId[BACKVOL]);
+	ret =  VL_IO;
+	goto out;
+    }
+    if (vldb_delete_entry(name)) {
+	mlog_log (MDEBVL, "VL_DeleteEntry failed to remove data\n");
+	ret =  VL_IO;
+	goto out;
+    }
+    
+    free(name);
+
+    vldb_flush();
+
+    ret = 0;
+    
+ out:
+    mlog_log (MDEBVL, "VL_DeleteEntry returns %d\n", ret);
+
+    return ret;
 }
 
 /*
@@ -130,28 +180,22 @@ VL_GetEntryByID(struct rx_call *call,
 		const int32_t voltype, 
 		vldbentry *entry) 
 {
-    struct disk_vlentry vl_entry;
-
-    vldb_debug ("VL_GetEntryByID (Volid=%d,Voltype=%d)\n", 
+    disk_vlentry diskentry;
+    char *name;
+    mlog_log (MDEBVL, "VL_GetEntryByID (Volid=%d,Voltype=%d)\n", 
 		  Volid, voltype);
-    
-    if (vldb_get_first_id_entry(vldb_get_id_hash(Volid),
-				voltype, &vl_entry) != 0)
+
+    if (vldb_id_to_name(Volid, &name))
 	return VL_NOENT;
 
-    while (1) {
-	/* Return entry if match found */
-	if (vl_entry.volumeId[voltype] == Volid) {
-	    make_vldb_from_vl(entry, &vl_entry);
-	    return 0;
-	}
-	
-	if (vl_entry.nextIdHash[voltype] == 0)
-	    break;
-	
-	vldb_read_entry(vl_entry.nextIdHash[voltype], &vl_entry);
-    }
-    return VL_NOENT;
+    if (vldb_read_entry(name, &diskentry) != 0)
+	return VL_NOENT;
+
+    vldb_disk_to_entry(&diskentry, entry);
+
+    free(name);
+
+    return 0;
 }
 
 /*
@@ -163,31 +207,21 @@ VL_GetEntryByName(struct rx_call *call,
 		  const char *volumename, 
 		  vldbentry *entry) 
 {
-    struct disk_vlentry vl_entry;
+    disk_vlentry diskentry;
 
-    vldb_debug ("VL_GetEntryByName %s\n", volumename) ;
+    mlog_log (MDEBVL, "VL_GetEntryByName (volumename = %s)\n", 
+		  volumename);
 
     if (isdigit(volumename[0])) {
 	return VL_GetEntryByID(call, atol(volumename), 0 /* XXX */, entry);
     }
 
-    if (vldb_get_first_name_entry(vldb_get_name_hash(volumename),
-				  &vl_entry) == 0) {
-	while (1) {
-	    /* Return entry if match found */
-	    if (strcmp(vl_entry.name, volumename) == 0) {
-		make_vldb_from_vl(entry, &vl_entry);
-		return 0;
-	    }
-	    
-	    if (vl_entry.nextNameHash == 0)
-		break;
-	    
-	    vldb_read_entry(vl_entry.nextNameHash, &vl_entry);
-	}
-    } 
+    if (vldb_read_entry(volumename, &diskentry) != 0)
+	return VL_NOENT;
 
-    return VL_NOENT;
+    vldb_disk_to_entry(&diskentry, entry);
+    
+    return 0;
 }
 
 /*
@@ -199,16 +233,18 @@ VL_GetNewVolumeId (struct rx_call *call,
 		   const int32_t bumpcount,
 		   int32_t *newvolumid)
 {
-    vldb_debug ("VL_GetNewVolumeId(bumpcount=%d)\n", bumpcount) ;
+    mlog_log (MDEBVL, "VL_GetNewVolumeId(bumpcount=%d)\n", bumpcount) ;
     
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
-    *newvolumid = vl_header.vital_header.MaxVolumeId;
-    vldb_debug ("   returning low volume id = %d\n", *newvolumid);
-    vl_header.vital_header.MaxVolumeId += bumpcount;
+    *newvolumid = vl_header.MaxVolumeId;
+    mlog_log (MDEBVL, "   returning low volume id = %d\n", *newvolumid);
+    vl_header.MaxVolumeId += bumpcount;
     vldb_write_header();
 
+    vldb_flush();
+
     return 0;
 }
 
@@ -223,7 +259,7 @@ VL_ReplaceEntry (struct rx_call *call,
 		 const vldbentry *newentry,
 		 const int32_t ReleaseType) 
 {
-    vldb_debug ("VL_ReplaceEntry\n") ;
+    mlog_log (MDEBVL, "VL_ReplaceEntry\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -242,7 +278,7 @@ VL_UpdateEntry (struct rx_call *call,
 		const VldbUpdateEntry *UpdateEntry,
 		const int32_t ReleaseType)
 {
-    vldb_debug ("VL_UpdateEntry\n") ;
+    mlog_log (MDEBVL, "VL_UpdateEntry\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -260,7 +296,7 @@ VL_SetLock (struct rx_call *call,
 	    const int32_t voltype,
 	    const int32_t voloper) 
 {
-    vldb_debug ("VL_SetLock\n") ;
+    mlog_log (MDEBVL, "VL_SetLock\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -278,7 +314,7 @@ VL_ReleaseLock (struct rx_call *call,
 		const int32_t voltype, 
 		const int32_t ReleaseType) 
 {
-    vldb_debug ("VL_ReleaseLock\n") ;
+    mlog_log (MDEBVL, "VL_ReleaseLock\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -298,7 +334,7 @@ VL_ListEntry (struct rx_call *call,
 	      int32_t *next_index, 
 	      vldbentry *entry) 
 {
-    vldb_debug ("VL_ListEntry\n") ;
+    mlog_log (MDEBVL, "VL_ListEntry\n") ;
     return VL_PERM ;
 }
 
@@ -312,7 +348,7 @@ VL_ListAttributes (struct rx_call *call,
 		   int32_t *nentries,
 		   bulkentries *blkentries) 
 {
-    vldb_debug ("VL_ListAttributes\n") ;
+    mlog_log (MDEBVL, "VL_ListAttributes\n") ;
     return VL_PERM ;
 }
 
@@ -325,7 +361,7 @@ VL_GetStats (struct rx_call *call,
 	     vldstats *stats,
 	     vital_vlheader *vital_header) 
 {
-    vldb_debug ("VL_GetStats") ;
+    mlog_log (MDEBVL, "VL_GetStats") ;
     return VL_PERM ;
 }
 
@@ -336,7 +372,7 @@ VL_GetStats (struct rx_call *call,
 int 
 VL_Probe(struct rx_call *call)
 {
-    vldb_debug ("VL_Probe\n") ;
+    mlog_log (MDEBVL, "VL_Probe\n") ;
     return 0;
 }
 
@@ -348,33 +384,56 @@ int
 VL_CreateEntryN(struct rx_call *call,
 		const nvldbentry *entry)
 {
-    int i;
-    struct vldbentry vldb_entry;
+    char *name;
+    disk_vlentry diskentry;
+    disk_vlentry tempentry;
 
-    vldb_debug ("VL_CreateEntryN\n") ;
+    mlog_log (MDEBVL, "VL_CreateEntryN (name=%s, ids=%d,%d,%d flags=%d)\n",
+	      entry->name,
+	      entry->volumeId[RWVOL],
+	      entry->volumeId[ROVOL],
+	      entry->volumeId[BACKVOL],
+	      entry->flags);
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
-    memset (&vldb_entry, 0, sizeof (vldb_entry));
 
-    strncpy(vldb_entry.name, entry->name, VLDB_MAXNAMELEN);
-    vldb_entry.volumeType = RWVOL;
-    vldb_entry.nServers = entry->nServers;
-
-    for (i = 0; i < MAXNSERVERS; i++) {
-	vldb_entry.serverNumber[i] = entry->serverNumber[i];
-	vldb_entry.serverPartition[i] = entry->serverPartition[i];
-	vldb_entry.serverFlags[i] = entry->serverFlags[i];
+    if ((vldb_id_to_name(entry->volumeId[RWVOL], &name) == 0) ||
+	(vldb_id_to_name(entry->volumeId[ROVOL], &name) == 0) ||
+	(vldb_id_to_name(entry->volumeId[BACKVOL], &name) == 0)) {
+	free(name);
+	mlog_log (MDEBVL, "VL_CreateEntryN: id exists\n");
+	return VL_NAMEEXIST;
     }
 
-    for (i = 0; i < MAXTYPES; i++)
-	vldb_entry.volumeId[i] = entry->volumeId[i];
+    if (vldb_read_entry(entry->name, &tempentry) == 0) {
+	mlog_log (MDEBVL, "VL_CreateEntryN: name exists\n");
+	return VL_NAMEEXIST;
+    }
 
-    vldb_entry.cloneId = entry->cloneId;
-    vldb_entry.flags = entry->flags;
+    vldb_nentry_to_disk(entry, &diskentry);
 
-    return VL_CreateEntry(call, &vldb_entry);
+    if (vldb_write_entry(&diskentry) != 0)
+	return VL_IO;
+
+    if (vldb_write_id(entry->name,
+		      entry->volumeId[RWVOL]) != 0)
+	return VL_IO; /* XXX rollback */
+
+    if (vldb_write_id(entry->name,
+		      entry->volumeId[ROVOL]) != 0)
+	return VL_IO; /* XXX rollback */
+
+    if (vldb_write_id(entry->name,
+		      entry->volumeId[BACKVOL]) != 0)
+	return VL_IO; /* XXX rollback */
+
+    vldb_free_diskentry(&diskentry);
+
+    vldb_flush();
+
+    return 0;
 }
 
 /*
@@ -387,35 +446,20 @@ VL_GetEntryByIDN(struct rx_call *call,
 		 const int32_t voltype,
 		 nvldbentry *entry)
 {
-    struct vldbentry vldb_entry;
-    int status, i;
-    int32_t type = voltype;
+    disk_vlentry diskentry;
+    char *name;
+    mlog_log (MDEBVL, "VL_GetEntryByIDN (Volid=%d,Voltype=%d)\n", 
+	      Volid, voltype);
 
-    vldb_debug ("VL_GetEntryByIDN (Volid=%d,Voltype=%d)\n", Volid, type);
+    if (vldb_id_to_name(Volid, &name))
+	return VL_NOENT;
 
-    memset (&vldb_entry, 0, sizeof(vldb_entry));
+    if (vldb_read_entry(name, &diskentry) != 0)
+	return VL_NOENT;
 
-    if (type == -1)
-	type = RWVOL;
+    vldb_disk_to_nentry(&diskentry, entry);
 
-    status = VL_GetEntryByID(call, Volid, type, &vldb_entry);
-
-    if (status)
-	return status;
-
-    strlcpy(entry->name, vldb_entry.name, VLDB_MAXNAMELEN);
-    entry->nServers = vldb_entry.nServers;
-    for (i = 0; i < MAXNSERVERS; i++) {
-	entry->serverNumber[i] = vldb_entry.serverNumber[i];
-	entry->serverPartition[i] = vldb_entry.serverPartition[i];
-	entry->serverFlags[i] = vldb_entry.serverFlags[i];
-    }
-
-    for (i = 0; i < MAXTYPES; i++)
-	entry->volumeId[i] = vldb_entry.volumeId[i];
-
-    entry->cloneId = vldb_entry.cloneId;
-    entry->flags = vldb_entry.flags;
+    free(name);
 
     return 0;
 }
@@ -429,35 +473,24 @@ VL_GetEntryByNameN(struct rx_call *call,
 		   const char *volumename, 
 		   nvldbentry *entry) 
 {
-    struct vldbentry vldb_entry;
-    int status, i;
+    disk_vlentry diskentry;
 
-    memset (&vldb_entry, 0, sizeof(vldb_entry));
+    mlog_log (MDEBVL, "VL_GetEntryByNameN (volumename = %s)\n", 
+	      volumename);
 
-    vldb_debug ("VL_GetEntryByNameN(volumename=%s)\n", volumename) ;
-    status = VL_GetEntryByName(call, volumename, &vldb_entry);
-
-    if (status)
-	return status;
-
-    memset (entry, 0, sizeof(*entry));
-    strlcpy(entry->name, vldb_entry.name, VLDB_MAXNAMELEN);
-    entry->nServers = vldb_entry.nServers;
-    for (i = 0; i < MAXNSERVERS; i++) {
-	entry->serverNumber[i] = vldb_entry.serverNumber[i];
-	entry->serverPartition[i] = vldb_entry.serverPartition[i];
-	entry->serverFlags[i] = vldb_entry.serverFlags[i];
+    if (isdigit(volumename[0])) {
+	return VL_GetEntryByIDN(call, atol(volumename), 0 /* XXX */, entry);
     }
 
-    for (i = 0; i < MAXTYPES; i++)
-	entry->volumeId[i] = vldb_entry.volumeId[i];
-
-    entry->cloneId = vldb_entry.cloneId;
-    entry->flags = vldb_entry.flags;
+    if (vldb_read_entry(volumename, &diskentry) != 0)
+	return VL_NOENT;
 
+    vldb_disk_to_nentry(&diskentry, entry);
+    
     return 0;
 }
 
+#ifdef notyet
 /*
  *
  */
@@ -467,11 +500,12 @@ VL_GetEntryByNameU(struct rx_call *call,
 		   const char *volumename, 
 		   uvldbentry *entry) 
 {
-    vldb_debug ("VL_GetEntryByNameU %s\n", volumename);
+    mlog_log (MDEBVL, "VL_GetEntryByNameU %s\n", volumename);
     memset(entry, 0, sizeof(*entry));
     
     return RXGEN_OPCODE;
 }
+#endif
 
 /*
  *
@@ -483,21 +517,19 @@ VL_ListAttributesN (struct rx_call *call,
 		   int32_t *nentries,
 		   nbulkentries *blkentries) 
 {
-    vldb_debug ("VL_ListAttributesN\n");
-    vldb_debug ("  attributes: Mask=(%d=", attributes->Mask);
+    mlog_log (MDEBVL, "VL_ListAttributesN\n");
+    mlog_log (MDEBVL, "  attributes: Mask=(%d=", attributes->Mask);
 
     if (attributes->Mask & VLLIST_SERVER)
-	vldb_debug  ("SERVER ");
+	mlog_log (MDEBVL, "SERVER ");
     if (attributes->Mask & VLLIST_PARTITION)
-	vldb_debug  ("PARTITION ");
-    if (attributes->Mask & VLLIST_VOLUMETYPE)
-	vldb_debug  ("VOLUMETYPE ");
+	mlog_log (MDEBVL, "PARTITION ");
     if (attributes->Mask & VLLIST_VOLUMEID)
-	vldb_debug  ("VOLUMEID ");
+	mlog_log (MDEBVL, "VOLUMEID ");
     if (attributes->Mask & VLLIST_FLAG)
-	vldb_debug  ("FLAG");
+	mlog_log (MDEBVL, "FLAG");
 
-    vldb_debug (") server=%d partition=%d volumetype=%d volumeid=%d flag=%d\n",
+    mlog_log (MDEBVL, ") server=%d partition=%d volumetype=%d volumeid=%d flag=%d\n",
 	   attributes->server,
 	   attributes->partition,
 	   attributes->volumetype,
@@ -512,6 +544,7 @@ VL_ListAttributesN (struct rx_call *call,
     return VL_PERM;
 }
 
+#ifdef notyet
 /*
  *
  */
@@ -522,12 +555,13 @@ VL_ListAttributesU(struct rx_call *call,
 		   int32_t *nentries,
 		   ubulkentries *blkentries) 
 {
-    vldb_debug ("VL_ListAttributesU\n") ;
+    mlog_log (MDEBVL, "VL_ListAttributesU\n") ;
     *nentries = 0;
     blkentries->len = 0;
     blkentries->val = NULL;
     return 0;
 }
+#endif
 
 /*
  *
@@ -539,7 +573,7 @@ VL_UpdateEntryByName(struct rx_call *call,
 		     const struct VldbUpdateEntry *UpdateEntry,
 		     const int32_t ReleaseType)
 {
-    vldb_debug ("VL_UpdateEntryByName (not implemented)\n");
+    mlog_log (MDEBVL, "VL_UpdateEntryByName (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -559,8 +593,8 @@ VL_GetAddrsU(struct rx_call *call,
 	     int32_t *nentries,
 	     bulkaddrs *addrs)
 {
-    vldb_debug ("VL_GetAddrsU (not implemented)\n");
-    return VL_PERM;
+    mlog_log (MDEBVL, "VL_GetAddrsU (not implemented)\n");
+    return RXGEN_OPCODE;
 }
 
 /*
@@ -573,7 +607,7 @@ VL_RegisterAddrs(struct rx_call *call,
 		 const int32_t spare,
 		 const bulkaddrs *addrs)
 {
-    vldb_debug ("VL_RegistersAddrs (not implemented)\n");
+    mlog_log (MDEBVL, "VL_RegistersAddrs (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -581,6 +615,7 @@ VL_RegisterAddrs(struct rx_call *call,
     return 0;
 }
 
+#ifdef notyet
 /*
  *
  */
@@ -589,14 +624,16 @@ int
 VL_CreateEntryU(struct rx_call *call,
 		const struct uvldbentry *newentry)
 {
-    vldb_debug ("VL_CreateEntryU (not implemented)\n");
+    mlog_log (MDEBVL, "VL_CreateEntryU (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
     return VL_PERM;
 }
+#endif
 
+#ifdef notyet
 /*
  *
  */
@@ -604,13 +641,14 @@ VL_CreateEntryU(struct rx_call *call,
 int
 VL_ReplaceEntryU(struct rx_call *call)
 {
-    vldb_debug ("VL_ReplaceEntryU (not implemented)\n");
+    mlog_log (MDEBVL, "VL_ReplaceEntryU (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
     return VL_PERM;
 }
+#endif
 
 /*
  *
@@ -620,10 +658,10 @@ int
 VL_ReplaceEntryN(struct rx_call *call,
 		 const int32_t Volid,
 		 const int32_t voltype,
-		 const struct vldbentry *newentry,
+		 const struct nvldbentry *newentry,
 		 const int32_t ReleaseType)
 {
-    vldb_debug ("VL_ReplaceEntryN (not implemented)\n");
+    mlog_log (MDEBVL, "VL_ReplaceEntryN (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -640,7 +678,7 @@ VL_ChangeAddrs(struct rx_call *call,
 	       const int32_t old_ip,
 	       const int32_t new_ip)
 {
-    vldb_debug ("VL_ChangeAddrs (not implemented)\n");
+    mlog_log (MDEBVL, "VL_ChangeAddrs (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -648,6 +686,7 @@ VL_ChangeAddrs(struct rx_call *call,
     return VL_PERM;
 }
 
+#ifdef notyet
 /*
  *
  */
@@ -655,21 +694,27 @@ VL_ChangeAddrs(struct rx_call *call,
 int
 VL_GetEntryByIDU(struct rx_call *call)
 {
-    vldb_debug ("VL_GetEntryByIDU (not implemented)\n");
+    mlog_log (MDEBVL, "VL_GetEntryByIDU (not implemented)\n");
     return VL_PERM;
 }
+#endif
 
 /*
  *
  */
 
 int
-VL_ListEntryN(struct rx_call *call)
+VL_ListEntryN(struct rx_call *call,
+	      int32_t previous_index,
+	      int32_t *count,
+	      int32_t *next_index,
+	      nvldbentry *entry)
 {
-    vldb_debug ("VL_ListEntryN (not implemented)\n");
+    mlog_log (MDEBVL, "VL_ListEntryN (not implemented)\n");
     return VL_PERM;
 }
 
+#ifdef notyet
 /*
  *
  */
@@ -677,9 +722,10 @@ VL_ListEntryN(struct rx_call *call)
 int
 VL_ListEntryU(struct rx_call *call)
 {
-    vldb_debug ("VL_ListEntryU (not implemented)\n");
+    mlog_log (MDEBVL, "VL_ListEntryU (not implemented)\n");
     return VL_PERM;
 }
+#endif
 
 /*
  *
@@ -690,13 +736,14 @@ VL_GetAddrs(struct rx_call *call,
 	    const int32_t handle,
 	    const int32_t spare,
 	    struct VL_Callback *spare3,
-	    const int32_t *nentries,
+	    int32_t *nentries,
 	    bulkaddrs *blkaddr)
 {
-    vldb_debug ("VL_GetAddrs (not implemented)\n");
+    mlog_log (MDEBVL, "VL_GetAddrs (not implemented)\n");
     return VL_PERM;
 }
 
+#ifdef notyet
 /*
  *
  */
@@ -704,7 +751,7 @@ VL_GetAddrs(struct rx_call *call,
 int
 VL_LinkedListN(struct rx_call *call)
 {
-    vldb_debug ("VL_LinkedListN (not implemented)\n");
+    mlog_log (MDEBVL, "VL_LinkedListN (not implemented)\n");
     return VL_PERM;
 }
 
@@ -715,11 +762,23 @@ VL_LinkedListN(struct rx_call *call)
 int
 VL_LinkedListU(struct rx_call *call)
 {
-    vldb_debug ("VL_LinkedListU (not implemented)\n");
+    mlog_log (MDEBVL, "VL_LinkedListU (not implemented)\n");
     return VL_PERM;
 }
+#endif
 
-
+int
+VL_ListAttributesN2(struct rx_call *call,
+		    const struct VldbListByAttributes *attributes,
+		    const char *volumename,
+		    const int32_t startindex,
+		    int32_t *nentries,
+		    nbulkentries *blkentries,
+		    int32_t *nextstartindex)
+{
+    mlog_log (MDEBVL, "VL_ListAttributesN2 (not implemented)\n");
+    return VL_PERM;
+}
 /*
  *
  */
@@ -729,29 +788,33 @@ static struct rx_service *ubikservice = NULL;
 
 static char *cell = NULL;
 static char *realm = NULL;
-static char *databasedir = NULL;
 static char *srvtab_file = NULL;
 static char *log_file = "syslog";
+static char *debug_levels = NULL;
 static int no_auth = 0;
+static int do_help = 0;
+static char *databasedir = NULL;
 static int do_create = 0;
-static int vlsrv_debug = 0;
 
 static struct agetargs args[] = {
-    {"create",  0, aarg_flag,      &do_create, "create new database"},
     {"cell",	0, aarg_string,    &cell, "what cell to use"},
     {"realm",	0, aarg_string,	  &realm, "what realm to use"},
-    {"prefix",'p', aarg_string,    &databasedir, "what dir to store the db"},
+    {"debug",  'd', aarg_string,  &debug_levels, "debug level"},
+    {"log",	'l',	aarg_string,	&log_file,
+     "where to write log (stderr, syslog (default), or path to file)"},
+    {"srvtab", 0, aarg_string,    &srvtab_file, "what srvtab to use"},
     {"noauth", 0,  aarg_flag,	  &no_auth, "disable authentication checks"},
-    {"debug", 'd', aarg_flag,      &vlsrv_debug, "output debugging"},
-    {"log",   'd', aarg_string,    &log_file, "log file"},
-    {"srvtab",'s', aarg_string,    &srvtab_file, "what srvtab to use"},
+    {"help",  'h', aarg_flag,      &do_help, "help"},
+    {"dbdir",  0, aarg_string,    &databasedir, "where to store the db"},
+    {"create",  0, aarg_flag,      &do_create, "create new database"},
     { NULL, 0, aarg_end, NULL }
 };
 
 static void
-usage(void)
+usage(int exit_code)
 {
-    aarg_printusage(args, NULL, "", AARG_AFSSTYLE);
+    aarg_printusage (args, NULL, "", AARG_GNUSTYLE);
+    exit (exit_code);
 }
 
 int
@@ -761,11 +824,10 @@ main(int argc, char **argv)
     int optind = 0;
     int ret;
     
-    setprogname (argv[0]);
+    set_progname (argv[0]);
 
-    if (agetarg (args, argc, argv, &optind, AARG_AFSSTYLE)) {
-	usage ();
-	return 1;
+    if (agetarg (args, argc, argv, &optind, AARG_GNUSTYLE)) {
+	usage (1);
     }
 
     argc -= optind;
@@ -776,29 +838,34 @@ main(int argc, char **argv)
 	return 1;
     }
 
-    if (vlsrv_debug)
-	vldb_setdebug (vlsrv_debug);
+    if (do_help)
+	usage(0);
 
     if (no_auth)
 	sec_disable_superuser_check ();
 
-    if (do_create) {
-	vldb_create (databasedir);
-	return 0;
-    }
-	
     method = log_open (getprogname(), log_file);
     if (method == NULL)
 	errx (1, "log_open failed");
     cell_init(0, method);
     ports_init();
 
+    mlog_loginit (method, milko_deb_units, MDEFAULT_LOG);
+
+    if (debug_levels)
+	mlog_log_set_level (debug_levels);
 
     if (cell)
 	cell_setthiscell (cell);
 
     network_kerberos_init (srvtab_file);
     
+    if (do_create) {
+	vldb_create (databasedir);
+	vldb_close();
+	return 0;
+    }
+
     vldb_init(databasedir);
 
     ret = network_init(htons(afsvldbport), "vl", VLDB_SERVICE_ID, 
