@@ -44,18 +44,46 @@
 #include <linux/file.h>
 #include <linux/smp_lock.h>
 #include <linux/smp.h>
+#include <linux/proc_fs.h>
 
-#ifdef RCSID
-RCSID("$Id: nnpfs_syscalls.c,v 1.92 2004/06/03 08:34:33 tol Exp $");
+#ifdef CONFIG_SECURITY
+#include <linux/security.h>
 #endif
 
-typedef int (*sys_afs_function)(int operation,
-				char *a_pathP,
-				int a_opcode,
-				struct ViceIoctl *a_paramsP,
-				int a_followSymlinks);
+#ifdef RCSID
+RCSID("$Id: nnpfs_syscalls.c,v 1.92.2.6 2004/06/24 09:06:43 lha Exp $");
+#endif
 
-typedef int (*sys_setgroups_function)(int, gid_t *);
+static struct proc_dir_entry *nnpfs_procfs_dir;
+
+#define VIOC_SYSCALL _IOW('C',1,void *)
+#define NNPFS_PROC_DIR "nnpfs"
+#define NNPFS_PROC_NODE "afs_ioctl"
+ 
+typedef struct afsprocdata {
+    unsigned long param4;
+    unsigned long param3;
+    unsigned long param2;
+    unsigned long param1;
+    unsigned long syscall;
+} afsprocdata;
+
+static int
+nnpfs_procfs_ioctl(struct inode *inode, struct file *file,
+		   unsigned int cmd, unsigned long arg);
+
+static struct file_operations nnpfs_procfs_fops = {
+    .ioctl = nnpfs_procfs_ioctl,
+};
+
+
+typedef asmlinkage long (*sys_afs_function)(int operation,
+					    char *a_pathP,
+					    int a_opcode,
+					    struct ViceIoctl *a_paramsP,
+					    int a_followSymlinks);
+
+typedef asmlinkage long (*sys_setgroups_function)(int, gid_t *);
 
 #ifdef ARLA_NR_setgroups16
 typedef int (*sys_setgroups16_function)(int, old_gid_t *);
@@ -84,15 +112,75 @@ static nnpfs_sys_call_function old_setgroups = NULL;
 static nnpfs_sys_call_function old_setgroups16 = NULL;
 #endif
 
+static int nnpfs_sec_registered = 0;
 
 #ifdef LINUX2_5    
 
 /* valid PAGs are [NNPFS_PAG_LLIM, NNPFS_PAG_ULIM) */
 
-#define NNPFS_PAG_LLIM 0x10000000UL
-#define NNPFS_PAG_ULIM 0xfffffff0UL
+#define NNPFS_PAG_LLIM 0x10000000
+#define NNPFS_PAG_ULIM 0x7fffffff
 #define NNPFS_PAG_NOTFOUND NNPFS_PAG_ULIM
 
+#ifdef CONFIG_SECURITY
+
+/*
+ * simple implementation, store pag in current->security
+ * XXX should be malloced, refcounted etc.
+ *
+ * this depends on us being able to register as primary LSM
+ * fallback on group list version
+ */
+
+static int
+nnpfs_sec_task_alloc(struct task_struct *p)
+{
+    nnpfs_pag_t pag = (nnpfs_pag_t)current->security;
+    if (pag)
+	p->security = (void *)pag;
+    return 0;
+}
+
+static void
+nnpfs_sec_task_free(struct task_struct *p)
+{
+    nnpfs_pag_t pag = (nnpfs_pag_t)p->security;
+    if (pag)
+	p->security = NULL;
+}
+
+static nnpfs_pag_t
+nnpfs_get_pag_sec(void)
+{
+    nnpfs_pag_t pag = (nnpfs_pag_t)current->security;
+    if (pag)
+	return pag;
+
+    return current->uid;
+}
+
+static int
+nnpfs_set_pag_sec(void)
+{
+    static nnpfs_pag_t pagnum = NNPFS_PAG_LLIM;
+
+    if (pagnum == NNPFS_PAG_ULIM)
+	return -ENOMEM;
+    /* pagnum = NNPFS_PAG_LLIM; */
+    
+    current->security = (void *)pagnum;
+    pagnum++;
+    return 0;
+}
+
+static struct security_operations nnpfs_sec_ops = {
+    .task_alloc_security = nnpfs_sec_task_alloc,
+    .task_free_security = nnpfs_sec_task_free,
+//    .task_reparent_to_init = nnpfs_sec_reparent_to_init,
+};
+
+#endif /* CONFIG_SECURITY */
+
 /* find pag index in group list, or return NNPFS_PAG_NOTFOUND */
 static int
 find_pag(struct group_info *gi)
@@ -115,8 +203,8 @@ find_pag(struct group_info *gi)
     return NNPFS_PAG_NOTFOUND;
 }
 
-nnpfs_pag_t
-nnpfs_get_pag(void)
+static nnpfs_pag_t
+nnpfs_get_pag_group(void)
 {
     struct group_info *gi = current->group_info;
     nnpfs_pag_t ret = current->uid;
@@ -130,7 +218,7 @@ nnpfs_get_pag(void)
 
     put_group_info(gi);
 
-    NNPFSDEB(XDEBSYS, ("nnpfs_get_pag: returning %u\n", ret));
+    NNPFSDEB(XDEBSYS, ("nnpfs_get_pag_group: returning %u\n", ret));
     return ret;
 }
 
@@ -182,7 +270,7 @@ store_pag(nnpfs_pag_t pagnum)
 }
 
 static int
-nnpfs_setpag_call(void)
+nnpfs_set_pag_group(void)
 {
     static gid_t pagnum = NNPFS_PAG_LLIM;
     int ret;
@@ -195,24 +283,47 @@ nnpfs_setpag_call(void)
     if (ret == 0)
 	pagnum++;
     else
-	NNPFSDEB(XDEBSYS, ("nnpfs_setpag_call: returning %u\n", ret));
+	NNPFSDEB(XDEBSYS, ("nnpfs_set_pag_group: returning %u\n", ret));
 
     return ret;
 }
 
+nnpfs_pag_t
+nnpfs_get_pag()
+{
+#ifdef CONFIG_SECURITY
+    if (nnpfs_sec_registered)
+	return nnpfs_get_pag_sec();
+#endif /* !CONFIG_SECURITY */
+
+    return nnpfs_get_pag_group();
+}
+
+static inline int
+nnpfs_setpag_call(void)
+{
+#ifdef CONFIG_SECURITY
+    if (nnpfs_sec_registered)
+	return nnpfs_set_pag_sec();
+#endif /* !CONFIG_SECURITY */
+
+    return nnpfs_set_pag_group();
+}
+
 /*
  * A wrapper around sys_setgroups that tries to preserve the pag.
  */
-static asmlinkage int
+static asmlinkage long
 nnpfs_setgroups (int gidsetsize, gid_t __user *usergrouplist)
 {
     sys_setgroups_function setgroups = (sys_setgroups_function)old_setgroups;
     nnpfs_pag_t current_pag = nnpfs_get_pag();
     gid_t grouplist[NGROUPS_SMALL];
-    int ret, i, n, offset;
+    int i, n, offset;
     int pag_in_orig = 0;
     int pag_in_new = 0;
-    
+    long ret;
+
     if (current_pag >= NNPFS_PAG_LLIM && current_pag < NNPFS_PAG_ULIM)
 	pag_in_orig = 1;
 
@@ -250,7 +361,7 @@ nnpfs_setgroups (int gidsetsize, gid_t __user *usergrouplist)
  * We need to fix it up too.
  */
 
-static asmlinkage int
+static asmlinkage long
 nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
 {
     /* We don't like 16-bit gids anyway */
@@ -259,13 +370,13 @@ nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
 
 #if 0
 /* This is not in sync with setgroups() above */
-static asmlinkage int
+static asmlinkage long
 nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
 {
 
     sys_setgroups16_function setgroups16 = (sys_setgroups16_function)old_setgroups16;
     nnpfs_pag_t current_pag = nnpfs_get_pag();
-    int ret;
+    long ret;
     int pag_in_orig = 0;
     
     if (current_pag >= NNPFS_PAG_LLIM && current_pag < NNPFS_PAG_ULIM)
@@ -398,16 +509,16 @@ nnpfs_setpag_call(void)
  * allow anyone, even root, from attaching to somebody else's pag.
  */
 
-static asmlinkage int
+static asmlinkage long
 nnpfs_setgroups (int gidsetsize, gid_t *grouplist)
 {
     sys_setgroups_function setgroups = (sys_setgroups_function)old_setgroups;
 
     if (nnpfs_is_pag ()) {
 	gid_t part1, part2;
-	int ret;
+	long ret;
 	/* Always save room for the PAG. */
-	if(gidsetsize + 2 >= NGROUPS)
+	if (gidsetsize + 2 >= NGROUPS)
 	    return -E2BIG;
 
 	part1 = current->groups[0];
@@ -418,14 +529,14 @@ nnpfs_setgroups (int gidsetsize, gid_t *grouplist)
 	       groups, i.e. if the user's group buffer was partially
 	       out of its address space. Prevent this trick from being
 	       used to store a fake PAG. */
-	    if(nnpfs_is_pag ())
+	    if (nnpfs_is_pag ())
 		store_pag (part1, part2);
 
 	    return ret;
 	}
 	return store_pag (part1, part2);
     } else {
-	int ret;
+	long ret;
 	ret = (*setgroups) (gidsetsize, grouplist);
 
 	/* Did a naughty superuser try to create a PAG? */
@@ -447,31 +558,30 @@ nnpfs_setgroups (int gidsetsize, gid_t *grouplist)
  * We need to fix it up too.
  */
 
-static asmlinkage int
+static asmlinkage long
 nnpfs_setgroups16 (int gidsetsize, old_gid_t *grouplist)
 {
     sys_setgroups16_function setgroups16 = (sys_setgroups16_function)old_setgroups16;
 
     if (nnpfs_is_pag ()) {
 	gid_t part1, part2;
-	int ret;
+	long ret;
 
-	if(gidsetsize + 2 >= NGROUPS)
+	if (gidsetsize + 2 >= NGROUPS)
 	    return -E2BIG;
 
 	part1 = current->groups[0];
 	part2 = current->groups[1];
 	ret = (*setgroups16) (gidsetsize, grouplist);
 	if (ret) {
-	    if(nnpfs_is_pag ())
+	    if (nnpfs_is_pag ())
 		store_pag (part1, part2);
 
 	    return ret;
 	}
 	return store_pag (part1, part2);
     } else {
-	int ret;
-	ret = (*setgroups16) (gidsetsize, grouplist);
+	long ret = (*setgroups16) (gidsetsize, grouplist);
 
 	/* Prevent attaching to somebody's PAG */
 	if (nnpfs_is_pag ()) {
@@ -741,14 +851,14 @@ user_path2dentry (struct nameidata *nd, char *user_path, int follow)
     return nd->dentry;
 }
 
-asmlinkage int
+asmlinkage long
 sys_afs_int (int operation,
 	     char *a_pathP,
 	     int a_opcode,
 	     struct ViceIoctl *a_paramsP,
 	     int a_followSymlinks)
 {
-    int error = 0;
+    long error = 0;
     struct ViceIoctl vice_ioctl; 
     struct nnpfs_message_pioctl msg;
     struct nnpfs_message_wakeup_data *msg2;
@@ -902,7 +1012,7 @@ sys_afs_int (int operation,
     if (dentry)
 	path_release(&nd);
 
-    NNPFSDEB(XDEBSYS, ("nnpfs_syscall returns error: %d\n", error));
+    NNPFSDEB(XDEBSYS, ("nnpfs_syscall returns error: %ld\n", error));
 
     NNPFSDEB(XDEBSYS, ("sys_afs kernel unlock; cpu: %d\n", smp_processor_id()));
     unlock_kernel();
@@ -911,7 +1021,7 @@ sys_afs_int (int operation,
 }
 
 #ifdef NEED_VICEIOCTL32
-asmlinkage int
+asmlinkage long
 sys32_afs (int operation,
 	   char *a_pathP,
 	   int a_opcode,
@@ -936,7 +1046,7 @@ sys32_afs (int operation,
 }
 #endif
 
-asmlinkage int
+asmlinkage long
 sys_afs (int operation,
 	 char *a_pathP,
 	 int a_opcode,
@@ -954,22 +1064,59 @@ sys_afs (int operation,
 		       a_followSymlinks);
 }
 
-void
-install_afs_syscall(void)
+static int
+nnpfs_procfs_ioctl(struct inode *inode, struct file *file,
+		   unsigned int cmd, unsigned long arg)
 {
-#ifndef HAVE_KERNEL_SYS_CALL_TABLE
-    if (nnpfs_fixup_syscall_lossage()) {
-	NNPFSDEB(XDEBSYS, ("nnpfs_afs_syscall failed\n"));
-	return;
+    afsprocdata args;
+    long ret;
+
+    if (cmd != VIOC_SYSCALL)
+	return -EINVAL;
+
+    if (copy_from_user(&args, (void *)arg, sizeof(args)))
+	return -EFAULT;
+    
+    ret = sys_afs((int)args.syscall,
+		  (char *)args.param1, (int)args.param2,
+		  (struct ViceIoctl *)args.param3, (int)args.param4);
+    return ret;
+}
+
+static int __init nnpfs_init_procfs(void)
+{
+    struct proc_dir_entry *entry;
+    
+    nnpfs_procfs_dir = proc_mkdir(NNPFS_PROC_DIR, proc_root_fs);
+    if (nnpfs_procfs_dir == NULL)
+	return -ENOMEM;
+    
+    nnpfs_procfs_dir->owner = THIS_MODULE;
+    
+    entry = create_proc_entry(NNPFS_PROC_NODE, 0666, nnpfs_procfs_dir);
+    if (entry == NULL) {
+	NNPFSDEB(XDEBSYS, ("nnpfs_init_procfs: no node\n"));
+	remove_proc_entry(NNPFS_PROC_DIR, proc_root_fs);
+	return -ENOMEM;
     }
-#endif
-    old_afs_syscall = sys_call_table[__NR_afs_syscall];
-    sys_call_table[__NR_afs_syscall] = 
-	(nnpfs_sys_call_function)&sys_afs;
-#ifdef NEED_VICEIOCTL32
-    old_afs_syscall32 = sys_call_table32[__NR_afs_syscall];
-    sys_call_table32[__NR_afs_syscall] = (uint32_t)&sys32_afs;
-#endif
+    
+    entry->owner = THIS_MODULE;
+    entry->proc_fops = &nnpfs_procfs_fops;
+    
+    NNPFSDEB(XDEBSYS, ("nnpfs_init_procfs: success\n"));
+
+    return 0;
+}
+
+static void __exit nnpfs_exit_procfs(void)
+{
+    remove_proc_entry(NNPFS_PROC_NODE, nnpfs_procfs_dir);
+    remove_proc_entry(NNPFS_PROC_DIR, proc_root_fs);
+}
+
+static void
+install_setgroups(void)
+{
     old_setgroups = sys_call_table[ARLA_NR_setgroups];
     sys_call_table[ARLA_NR_setgroups] = 
 	(nnpfs_sys_call_function)&nnpfs_setgroups;
@@ -980,9 +1127,45 @@ install_afs_syscall(void)
 #endif
 }
 
+
+void
+install_afs_syscall(void)
+{
+#ifdef CONFIG_SECURITY
+    if (register_security(&nnpfs_sec_ops))
+	NNPFSDEB(XDEBSYS,
+		 ("install_afs_syscall: nnpfs_init_sec failed\n"));
+    else
+	nnpfs_sec_registered = 1;
+#endif
+
+    nnpfs_init_procfs();
+
+#ifndef HAVE_KERNEL_SYS_CALL_TABLE
+    if (nnpfs_fixup_syscall_lossage()) {
+	NNPFSDEB(XDEBSYS,
+		 ("install_afs_syscall: no syscalltable found\n"));
+	return;
+    }
+#endif
+
+    if (!nnpfs_sec_registered)
+	install_setgroups();	
+
+    old_afs_syscall = sys_call_table[__NR_afs_syscall];
+    sys_call_table[__NR_afs_syscall] = 
+	(nnpfs_sys_call_function)&sys_afs;
+#ifdef NEED_VICEIOCTL32
+    old_afs_syscall32 = sys_call_table32[__NR_afs_syscall];
+    sys_call_table32[__NR_afs_syscall] = (uint32_t)&sys32_afs;
+#endif
+}
+
 void
 restore_afs_syscall (void)
 {
+    nnpfs_exit_procfs();
+
     if (old_afs_syscall) {
 	sys_call_table[__NR_afs_syscall] = old_afs_syscall;
 	old_afs_syscall = NULL;
@@ -993,6 +1176,13 @@ restore_afs_syscall (void)
 	old_afs_syscall32 = 0;
     }
 #endif
+
+#ifdef CONFIG_SECURITY
+    if (nnpfs_sec_registered)
+	if (unregister_security(&nnpfs_sec_ops))
+	    printk(KERN_EMERG "nnpfs_exit_sec: couldn't unregister\n");
+#endif /* !CONFIG_SECURITY */
+
     if (old_setgroups) {
 	sys_call_table[ARLA_NR_setgroups] = old_setgroups;
 	old_setgroups = NULL;
@@ -1004,4 +1194,3 @@ restore_afs_syscall (void)
     }
 #endif
 }
-
