@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 2003 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #include "rxkad_locl.h"
 
-RCSID("$Id: rxk_locl.c,v 1.8 2000/10/03 00:38:38 lha Exp $");
+RCSID("$Id: rxk_locl.c,v 1.13 2004/01/30 15:41:52 lha Exp $");
 
 /* The header checksum is the last 16 bits of this struct after
  * encryption. Note that only the last 8 bytes change per packet. */
@@ -46,16 +46,16 @@ struct header_data {
 
 struct const_header_data {
   /* Data that is constant per connection */
-  u_int32 epoch;
-  u_int32 cid;
-  u_int32 zero;
-  u_int32 security_index;
+  uint32_t epoch;
+  uint32_t cid;
+  uint32_t zero;
+  uint32_t security_index;
 };
 
 struct variable_header_data {
   /* Data that changes per packet */
-  u_int32 call_number;
-  u_int32 channel_and_seq;
+  uint32_t call_number;
+  uint32_t channel_and_seq;
 };
 
 /* To create a 16 bit packet header checksum we first create an iv
@@ -63,22 +63,22 @@ struct variable_header_data {
  */
 void
 rxkad_calc_header_iv(const struct rx_connection *conn,
-		     const int32 *sched,
-		     const des_cblock *in_iv,
-		     u_int32 *out_iv)
+		     const int32_t *sched,
+		     char *in_iv,
+		     uint32_t *out_iv)
 {
   struct const_header_data h;
-  u_int32 *t;
+  uint32_t *t;
 
   h.epoch = htonl(conn->epoch);
   h.cid = htonl(conn->cid & RX_CIDMASK);
   h.zero = 0;
   h.security_index = htonl(conn->securityIndex);
 
-  t = (u_int32 *)in_iv;		/* memcpy(out_iv, in_iv, 8); */
+  t = (uint32_t *)in_iv;		/* memcpy(out_iv, in_iv, 8); */
   out_iv[0] = t[0];
   out_iv[1] = t[1];
-  fc_cbc_encrypt(&h, &h, sizeof(h), sched, out_iv, ENCRYPT);
+  fc_cbc_encrypt(&h, &h, sizeof(h), sched, out_iv, FC_ENCRYPT);
   /* Extract last 8 bytes as iv */
   assert(out_iv[0] == h.zero);
   /* out_iv[0] = h.zero; */
@@ -93,11 +93,11 @@ rxkad_calc_header_iv(const struct rx_connection *conn,
 static
 int
 rxkad_cksum_header(const struct rx_packet *packet,
-		   const int32 *sched,
+		   const int32_t *sched,
 		   const unsigned int *iv)
 {
   struct variable_header_data h;
-  u_int32 t;
+  uint32_t t;
 
   /* Collect selected packet fields */
   h.call_number = htonl(packet->header.callNumber);
@@ -108,7 +108,7 @@ rxkad_cksum_header(const struct rx_packet *packet,
   /* Encrypt selected fields (this is hand rolled CBC mode) */
   h.call_number     ^= iv[0];
   h.channel_and_seq ^= iv[1];
-  fc_ecb_encrypt(&h, &h, sched, ENCRYPT);
+  fc_ecb_encrypt(&h, &h, sched, FC_ENCRYPT);
 
   /* Select 16 bits that are now dependent on all selected packet fields */
   t = (ntohl(h.channel_and_seq) >> 16) & 0xffff;
@@ -120,11 +120,11 @@ rxkad_cksum_header(const struct rx_packet *packet,
 
 /* Checksum a rxkad_response, this checksum is buried within the
  * encrypted part of the response but covers the entire response. */
-u_int32
+uint32_t
 rxkad_cksum_response(rxkad_response *r)
 {
   u_char *t;
-  u_int32 cksum = 1000003;
+  uint32_t cksum = 1000003;
   
   for (t = (u_char *)r; t < (u_char*)(r + 1); t++)
     cksum = *t + cksum * 0x10204081;
@@ -149,9 +149,9 @@ rxkad_prepare_packet(struct rx_packet *pkt,
   
   if (level != rxkad_clear)
     {
-      u_int32 *data = (u_int32 *) rx_DataOf(pkt);
-      u_int32 t;
-      int32 code = 0;
+      uint32_t *data = (uint32_t *) rx_DataOf(pkt);
+      uint32_t t;
+      int32_t code = 0;
 
       assert(pkt->wirevec[1].iov_len >= 4);
 
@@ -175,7 +175,7 @@ rxkad_prepare_packet(struct rx_packet *pkt,
   	
 	/* Encrypt security header (4 bytes) and the next 4 bytes */
  	assert(pkt->wirevec[1].iov_len >= 8);
-	fc_ecb_encrypt(data, data, k->keysched, ENCRYPT);
+	fc_ecb_encrypt(data, data, k->keysched, FC_ENCRYPT);
 	break;
 
       case rxkad_crypt:
@@ -190,7 +190,7 @@ rxkad_prepare_packet(struct rx_packet *pkt,
  	rx_SetDataSize(pkt, len); /* Set extended packet length */
  
  	assert((len % 8) == 0);
- 	code = rxkad_EncryptPacket(con, k->keysched,(u_int32*)k->key, len,pkt);
+ 	code = rxkad_EncryptPacket(con, k->keysched,(uint32_t*)k->key.data, len,pkt);
 	break;
 
       default:
@@ -224,18 +224,18 @@ rxkad_check_packet(struct rx_packet *pkt,
   else
     {
       u_int len;		/* Real packet length */
-      u_int32 *data = (u_int32 *) rx_DataOf(pkt);
-      u_int32 t;
-      int32 code;
+      uint32_t *data = (uint32_t *) rx_DataOf(pkt);
+      uint32_t t;
+      int32_t code;
 
       switch (level) {
       case rxkad_auth:
 	assert(rx_Contiguous(pkt) >= 8);
-	fc_ecb_encrypt(data, data, k->keysched, DECRYPT);
+	fc_ecb_encrypt(data, data, k->keysched, FC_DECRYPT);
 	break;
 
       case rxkad_crypt:
-	code = rxkad_DecryptPacket(con, k->keysched,(u_int32*)k->key, xlen, pkt);
+	code = rxkad_DecryptPacket(con, k->keysched,(uint32_t*)k->key.data, xlen, pkt);
 	if (code)
 	  return code;
 	break;
