@@ -18,7 +18,7 @@ main(int argc, char **argv)
 {
     int fd;
 
-    set_progname (argv[0]);
+    setprogname (argv[0]);
 
     if (!k_hasafs())
 	errx (1, "no afs");
@@ -45,7 +45,7 @@ main(int argc, char **argv)
 int
 main (int argc, char **argv)
 {
-    set_progname (argv[0]);
+    setprogname (argv[0]);
 
     errx (1, "no kafs");
 }
