@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2000 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -53,12 +53,14 @@
 #include <err.h>
 #include <roken.h>
 
-RCSID("$Id: invalidate-file.c,v 1.3.2.1 2002/07/11 14:01:01 lha Exp $");
+RCSID("$Id: invalidate-file.c,v 1.5 2002/07/15 12:45:20 lha Exp $");
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *)-1)
 #endif
 
+#ifdef KERBEROS
+
 static void
 create_write_file (char *filename)
 {
@@ -169,7 +171,9 @@ main(int argc, char **argv)
     
     setprogname (argv[0]);
 
+#ifdef KERBEROS
     if (!k_hasafs())
+#endif
 	exit (1);
     
     create_write_file (filename);
@@ -188,3 +192,14 @@ main(int argc, char **argv)
     
     return 0;
 }
+
+#else
+
+int
+main(int argc, char **argv)
+{
+    setprogname (argv[0]);
+
+    errx (1, "no kafs support");
+}
+#endif
