@@ -48,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: large-dir2.c,v 1.3.2.1 2002/07/11 14:01:04 lha Exp $");
+RCSID("$Id: large-dir2.c,v 1.5 2002/09/17 18:27:11 lha Exp $");
 #endif
 
 static int
@@ -115,7 +115,7 @@ creat_files (const char *dirname, int count)
 static void
 usage (int ret)
 {
-    fprintf (stderr, "%s directory number-of-files\n", __progname);
+    fprintf (stderr, "%s directory number-of-files\n", getprogname());
     exit (ret);
 }
 
