@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -48,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: read-vs-mmap.c,v 1.9.2.1 2002/07/11 14:01:09 lha Exp $");
+RCSID("$Id: read-vs-mmap.c,v 1.12 2002/07/15 12:45:29 lha Exp $");
 #endif
 
 static int debug = 0;
@@ -62,7 +62,7 @@ generate_file (const char *filename, int randomp, size_t sz)
 
     buf = malloc (sz);
     if (buf == NULL)
-	err (1, "malloc %u", sz);
+	err (1, "malloc %u", (unsigned)sz);
 
     fd = open (filename, O_WRONLY | O_CREAT, 0666);
     if (fd < 0)
@@ -81,16 +81,20 @@ generate_file (const char *filename, int randomp, size_t sz)
     free (buf);
 }
 
-static char *
+static unsigned char *
 read_file (int fd, size_t sz)
 {
-    char *buf;
+    unsigned char *buf;
+    ssize_t ret;
 
     buf = malloc (sz);
     if (buf == NULL)
-	err (1, "malloc %u", sz);
-    if (read (fd, buf, sz) != sz)
-	err (1, "read");
+	err (1, "malloc %u", (unsigned)sz);
+    ret = read(fd, buf, sz);
+    if(ret < 0)
+      err(1, "read");
+    if(ret != sz)
+      errx(1, "short read %d < %u", (int)ret, (unsigned)sz);
     return buf;
 }
 
@@ -121,7 +125,7 @@ print_area (unsigned char *ptr,  size_t len)
 static int
 do_test (int randomp)
 {
-    char *malloc_buf;
+    unsigned char *malloc_buf;
     void *mmap_buf;
     int fd;
     const char *file = "foo";
