@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2000 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -62,7 +62,7 @@
 #define MAP_FAILED ((void *)-1)
 #endif
 
-RCSID("$Id: write-closed2.c,v 1.5.2.1 2002/07/11 14:01:20 lha Exp $");
+RCSID("$Id: write-closed2.c,v 1.7 2002/07/15 12:45:42 lha Exp $");
 
 static int
 set_acl (char *dir)
@@ -75,7 +75,11 @@ set_acl (char *dir)
     a_params.in       = foo;
     a_params.out      = NULL;
 
+#ifdef KERBEROS
     return k_pioctl (dir, VIOCSETAL, &a_params, 1);
+#else
+    return -1;
+#endif
 }
 
 static void
@@ -122,7 +126,9 @@ main(int argc, char **argv)
 
     setprogname (argv[0]);
 
+#ifdef KERBEROS
     if (!k_hasafs())
+#endif
 	errx (1, "!k_hasafs");
 
     if (argc != 1 && argc != 2)
