@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,10 +35,10 @@
  * list handling functions
  */
 
-/* $Id: list.h,v 1.9 1999/01/11 02:24:33 rb Exp $ */
+/* $Id: list.h,v 1.13 2003/01/10 12:38:43 lha Exp $ */
 
-#ifndef _LIST_
-#define _LIST_
+#ifndef _ARLAUTIL_LIST_H
+#define _ARLAUTIL_LIST_H 1
 
 #include "bool.h"
 #include <roken.h>
@@ -67,6 +62,8 @@ typedef struct list List;
 
 List *listnew (void);
 
+void listfree(List *);
+
 Listitem *listaddhead (List *list, void *data);
 
 Listitem *listaddtail (List *list, void *data);
@@ -130,4 +127,4 @@ listnextp(Listitem *item)
     return (Bool)(item->next != NULL);
 }
 
-#endif /* _LIST_ */
+#endif /* _ARLAUTIL_LIST_H */
